@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: apic.cc,v 1.54.2.1 2005/07/07 08:21:59 vruppert Exp $
+// $Id: apic.cc,v 1.70 2005/12/26 19:42:09 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -32,20 +32,17 @@
 #define LOG_THIS this->
 
 bx_generic_apic_c *apic_index[APIC_MAX_ID];
-bx_local_apic_c *local_apic_index[BX_LOCAL_APIC_NUM];
+bx_local_apic_c *local_apic_index[BX_NUM_LOCAL_APICS];
 
-#define LOCAL_APIC_ALL_MASK ((1<<BX_LOCAL_APIC_NUM) - 1)
+#define LOCAL_APIC_ALL_MASK ((1<<BX_NUM_LOCAL_APICS) - 1)
 
-bx_generic_apic_c::bx_generic_apic_c ()
+bx_generic_apic_c::bx_generic_apic_c()
 {
   id = APIC_UNKNOWN_ID;
   put("APIC?");
   settype(APICLOG);
 }
 
-// init is called during RESET and when an INIT message is delivered.
-void bx_generic_apic_c::init () { }
-
 void bx_generic_apic_c::set_base (bx_address newbase)
 {
   newbase &= (~0xfff);
@@ -71,7 +68,7 @@ void bx_generic_apic_c::set_id (Bit8u newid)
   }
 }
 
-void bx_generic_apic_c::reset_all_ids ()
+void bx_generic_apic_c::reset_all_ids(void)
 {
   for (int i=0; i<APIC_MAX_ID; i++) {
     if (apic_index[i]) 
@@ -79,17 +76,11 @@ void bx_generic_apic_c::reset_all_ids ()
   }
 }
 
-char *bx_generic_apic_c::get_name ()
-{
-  BX_PANIC(("get_name called on bx_generic_apic_c base class"));
-  return NULL;
-}
-
 bx_bool bx_generic_apic_c::is_selected (bx_address addr, Bit32u len)
 {
-  if ((addr & ~0xfff) == get_base ()) {
+  if ((addr & ~0xfff) == get_base()) {
     if (((addr & 0xf) != 0) || (len != 4))
-      BX_INFO(("warning: misaligned or wrong-size APIC access. addr=" FMT_ADDRX " len=%d", addr, len));
+      BX_INFO(("warning: misaligned or wrong-size APIC access. len=%d, addr=" FMT_ADDRX "", len, addr));
     return 1;
   }
   return 0;
@@ -123,16 +114,17 @@ void bx_generic_apic_c::read (Bit32u addr, void *data, unsigned len)
 /* apic_mask is the bitmask of apics allowed to arbitrate here */
 int bx_generic_apic_c::apic_bus_arbitrate(Bit32u apic_mask)
 {
-  int winning_apr = 0, winning_id = 0, __apr, i;
-  for (i = 0; i < BX_LOCAL_APIC_NUM; i++) {
+  int winning_arb_id = -1, winning_id = -1, __arb_id, i;
+  for (i = 0; i < BX_NUM_LOCAL_APICS; i++) {
     if (apic_mask & (1<<i)) {
-      __apr = local_apic_index[i]->get_apr();
-      if (__apr > winning_apr) {
-        winning_apr = __apr;
+      __arb_id = local_apic_index[i]->get_arb_id();
+      if (__arb_id > winning_arb_id) {
+        winning_arb_id = __arb_id;
 	winning_id = i;
       }
     }
   }
+  BX_ASSERT(winning_id >= 0);
   return winning_id;
 }
  
@@ -140,10 +132,10 @@ int bx_generic_apic_c::apic_bus_arbitrate(Bit32u apic_mask)
 int bx_generic_apic_c::apic_bus_arbitrate_lowpri(Bit32u apic_mask)
 {
   // XXX initial winning_apr value, the datasheets say 15
-  int winning_apr = APIC_MAX_ID, winning_id = 0 , __apr, i;
-  for (i = 0; i < BX_LOCAL_APIC_NUM; i++) {
+  int winning_apr = APIC_MAX_ID, winning_id = 0, __apr, i;
+  for (i = 0; i < BX_NUM_LOCAL_APICS; i++) {
     if (apic_mask & (1<<i)) {
-      __apr = local_apic_index[i]->get_apr_lowpri();
+      __apr = local_apic_index[i]->get_apr();
       if (__apr < winning_apr) {
 	winning_apr = __apr;
 	winning_id = i;
@@ -155,26 +147,21 @@ int bx_generic_apic_c::apic_bus_arbitrate_lowpri(Bit32u apic_mask)
 
 void bx_generic_apic_c::arbitrate_and_trigger(Bit32u deliver_bitmask, Bit32u vector, Bit8u trigger_mode)
 {
-  int trigger_order[BX_LOCAL_APIC_NUM], winner, i, j;
-#define TERMINATE_MAGIK	0x5a
+  int trigger_order[BX_NUM_LOCAL_APICS], winner, i, j = 0;
 
   /* bus arbitrate ... */
-  for (i = 0, j = 0; i < BX_LOCAL_APIC_NUM; i++) {
+  for (i = 0; i < BX_NUM_LOCAL_APICS; i++) {
     if (deliver_bitmask & (1<<i)) {
       winner = apic_bus_arbitrate(deliver_bitmask);
       local_apic_index[winner]->adjust_arb_id(winner);
       trigger_order[j++] = winner;
+      deliver_bitmask &= ~(1<<i);
     }
   }
-  
-  if (j < BX_LOCAL_APIC_NUM)
-    trigger_order[j] = TERMINATE_MAGIK;
 
-  i = 0;
-  do {
+  for (i = 0; i < j; i++) {
     local_apic_index[trigger_order[i]]->trigger_irq(vector, trigger_order[i], trigger_mode);
-    i++;
-  } while (trigger_order[i] != TERMINATE_MAGIK);
+  }
 }
  
 void bx_generic_apic_c::arbitrate_and_trigger_one(Bit32u deliver_bitmask, Bit32u vector, Bit8u trigger_mode)
@@ -184,24 +171,34 @@ void bx_generic_apic_c::arbitrate_and_trigger_one(Bit32u deliver_bitmask, Bit32u
   local_apic_index[winner]->trigger_irq(vector, winner, trigger_mode);
 }
 
+#ifdef BX_IMPLEMENT_XAPIC
+  #define APIC_BROADCAST_PHYSICAL_DESTINATION_MODE 0xff
+#else
+  #define APIC_BROADCAST_PHYSICAL_DESTINATION_MODE 0x0f
+#endif
+
 Bit32u bx_generic_apic_c::get_delivery_bitmask (Bit8u dest, Bit8u dest_mode)
 {
-  int mask = 0;
+  Bit32u mask = 0;
   if (dest_mode == 0) {
     // physical 
     if (dest < APIC_MAX_ID)
-      mask = 1<<dest;
-    else if (dest == 0xff) {
-      // physical destination 0xff means everybody. only local APICs can
-      // send this.
-      BX_ASSERT (get_type () == APIC_TYPE_LOCAL_APIC);
-      mask = LOCAL_APIC_ALL_MASK; 
-    } else BX_PANIC(("bx_generic_apic_c::deliver: illegal physical destination %02x", dest));
+      mask = 1 << dest;
+    else {
+      if (dest == APIC_BROADCAST_PHYSICAL_DESTINATION_MODE)
+      {
+        // physical destination 0xff means everybody. only local APICs can
+        // send this.
+        BX_ASSERT (get_type() == APIC_TYPE_LOCAL_APIC);
+        mask = LOCAL_APIC_ALL_MASK; 
+      } 
+      else BX_PANIC(("bx_generic_apic_c::deliver: illegal physical destination %02x", dest));
+    }
   } else {
     // logical destination. call match_logical_addr for each local APIC.
     if (dest == 0) return 0;
-    for (int i=0; i<BX_LOCAL_APIC_NUM; i++) {
-      if (local_apic_index[i]->match_logical_addr(dest))
+    for (int i=0; i<BX_NUM_LOCAL_APICS; i++) {
+      if (local_apic_index[i] && local_apic_index[i]->match_logical_addr(dest))
         mask |= (1<<i);
     }
   }
@@ -218,15 +215,22 @@ bx_bool bx_generic_apic_c::deliver (Bit8u dest, Bit8u dest_mode, Bit8u delivery_
   Bit32u deliver_bitmask = get_delivery_bitmask (dest, dest_mode);
   // arbitrate by default
   int arbitrate = 1;
-  int broadcast = (dest == LOCAL_APIC_ALL_MASK); // all local apics
+  int broadcast = (deliver_bitmask == LOCAL_APIC_ALL_MASK); // all local apics
   bx_bool once = 0;
   int i;
 
+  // prune nonexistents apics from list
+  for (int bit=0; bit<BX_NUM_LOCAL_APICS; bit++)
+  {
+    if (!local_apic_index[bit]) deliver_bitmask &= ~(1<<bit);
+  }
+
   // mask must include ONLY local APICs, or we will have problems.
   if (!deliver_bitmask) {
     BX_PANIC(("deliver failed for vector %02x: no APICs in destination bitmask", vector));
     return 0;
   }
+
   switch (delivery_mode) {
     case APIC_DM_LOWPRI:
       /* fall through, we've already done low priority arbitration */
@@ -242,13 +246,13 @@ bx_bool bx_generic_apic_c::deliver (Bit8u dest, Bit8u dest_mode, Bit8u delivery_
       // bx_local_apic_c::deliver
 
       // normal INIT IPI sent to processors
-      for (i = 0; i < BX_LOCAL_APIC_NUM; i++) {
+      for (i = 0; i < BX_NUM_LOCAL_APICS; i++) {
         if (deliver_bitmask & (1<<i)) local_apic_index[i]->init();
       }
       return 1;
     
     case APIC_DM_EXTINT:
-      for (i = 0; i < BX_LOCAL_APIC_NUM; i++)
+      for (i = 0; i < BX_NUM_LOCAL_APICS; i++)
         if (deliver_bitmask & (1<<i))
 	  local_apic_index[i]->bypass_irr_isr = 1;
        break;
@@ -273,7 +277,7 @@ bx_bool bx_generic_apic_c::deliver (Bit8u dest, Bit8u dest_mode, Bit8u delivery_
     if (arbitrate)
       arbitrate_and_trigger_one(deliver_bitmask, vector, trig_mode);
     else {
-      for (int i = 0; i < BX_LOCAL_APIC_NUM; i++) {
+      for (int i = 0; i < BX_NUM_LOCAL_APICS; i++) {
         if (deliver_bitmask & (1<<i)) {
           local_apic_index[i]->trigger_irq(vector, i, trig_mode);
           break;
@@ -284,7 +288,7 @@ bx_bool bx_generic_apic_c::deliver (Bit8u dest, Bit8u dest_mode, Bit8u delivery_
     if (arbitrate && !broadcast)
       arbitrate_and_trigger(deliver_bitmask, vector, trig_mode);
     else {
-      for (int i = 0; i < BX_LOCAL_APIC_NUM; i++) {
+      for (int i = 0; i < BX_NUM_LOCAL_APICS; i++) {
         if (deliver_bitmask & (1<<i))
           local_apic_index[i]->trigger_irq(vector, i, trig_mode);
       }
@@ -318,7 +322,7 @@ Bit32u bx_local_apic_c::get_delivery_bitmask (Bit8u dest, Bit8u dest_mode)
   }
 
   // prune nonexistents apics from list
-  for (int bit=0; bit<BX_LOCAL_APIC_NUM; bit++)
+  for (int bit=0; bit<BX_NUM_LOCAL_APICS; bit++)
   {
     if (!local_apic_index[bit]) mask &= ~(1<<bit);
   }
@@ -346,11 +350,13 @@ bx_bool bx_local_apic_c::deliver (Bit8u dest, Bit8u dest_mode, Bit8u delivery_mo
     // look for the focus processor.
     dest = is_focus(vector) ? get_id() : 0;
     if (dest) break;
-    for (int i = 0; i < BX_LOCAL_APIC_NUM; i++) {
-      if (local_apic_index[i]->is_focus(vector)) {
-        found_focus = 1;
-        dest = i;
-        break;	// stop scanning
+    for (int i = 0; i < BX_NUM_LOCAL_APICS; i++) {
+      if (local_apic_index[i]) {
+        if (local_apic_index[i]->is_focus(vector)) {
+          found_focus = 1;
+          dest = i;
+          break;	// stop scanning
+        }
       }
     }
     if (!found_focus) dest = apic_bus_arbitrate_lowpri(0xff);
@@ -366,14 +372,15 @@ bx_bool bx_local_apic_c::deliver (Bit8u dest, Bit8u dest_mode, Bit8u delivery_mo
       // arbitration ID to their APIC ID. Not supported by Pentium 4
       // and Intel Xeon processors.
       BX_INFO (("INIT with Level&Deassert: synchronize arbitration IDs"));
-      for (bit=0; bit<BX_LOCAL_APIC_NUM; bit++)
-        local_apic_index[bit]->set_arb_id(local_apic_index[bit]->get_id());
+      for (bit=0; bit<BX_NUM_LOCAL_APICS; bit++)
+        if (local_apic_index[bit])
+          local_apic_index[bit]->set_arb_id(local_apic_index[bit]->get_id());
       return 1;
     }
     break; // we'll fall through to generic_deliver:case INIT
 
   case APIC_DM_SIPI:  // Start Up (SIPI, local apic only)
-    for (bit=0; bit<BX_LOCAL_APIC_NUM; bit++) {
+    for (bit=0; bit<BX_NUM_LOCAL_APICS; bit++) {
       if (deliver_bitmask & (1<<bit))
         local_apic_index[bit]->startup_msg(vector);
     }
@@ -388,39 +395,51 @@ bx_bool bx_local_apic_c::deliver (Bit8u dest, Bit8u dest_mode, Bit8u delivery_mo
 }
 
 bx_local_apic_c::bx_local_apic_c(BX_CPU_C *mycpu)
-  : bx_generic_apic_c (), cpu(mycpu), cpu_id(cpu->which_cpu())
+  : bx_generic_apic_c(), cpu(mycpu), cpu_id(cpu->which_cpu())
 {
-  reset ();
+  reset();
+
+  // KPL: Register a non-active timer for use when the timer is started.
+  timer_handle = bx_pc_system.register_timer_ticks(this,
+            BX_CPU(0)->local_apic.periodic_smf, 0, 0, 0, "lapic");
+	
   INTR = 0;
 }
 
-void bx_local_apic_c::set_arb_id (int new_arb_id)
+Bit32u bx_local_apic_c::get_arb_id (void)
+{
+  return arb_id;
+}
+
+void bx_local_apic_c::set_arb_id (Bit32u new_arb_id)
 {
   BX_DEBUG (("set arbitration ID to %d", new_arb_id));
   arb_id = new_arb_id;
 }
 
-void bx_local_apic_c::reset () 
+void bx_local_apic_c::reset() 
 {
   /* same as INIT but also sets arbitration ID and APIC ID */
-  init ();
+  init();
   arb_id = id;
 }
 
-void bx_local_apic_c::init ()
+void bx_local_apic_c::init()
 {
   int i;
 
-  bx_generic_apic_c::init ();
+  bx_generic_apic_c::init();
 
   BX_INFO(("local apic in %s initializing",
       (cpu && cpu->name) ? cpu->name : "?"));
-  local_apic_index[id] = this;
+
+  if (id!=APIC_UNKNOWN_ID)
+      local_apic_index[id] = this;
 
   // default address for a local APIC, can be moved
   base_addr = APIC_BASE_ADDR;
   bypass_irr_isr = 0;
-  error_status = 0;
+  error_status = shadow_error_status = 0;
   log_dest = 0;
   dest_format = 0xf;
   icr_hi = 0;
@@ -436,21 +455,24 @@ void bx_local_apic_c::init ()
   timer_divide_factor = 1;
   timer_initial = 0;
   timer_current = 0;
-  timer_active = 0;
+
+  if (timer_active) {
+    bx_pc_system.deactivate_timer(timer_handle);
+    timer_active = 0;
+  }
 
   for (i=0; i<APIC_LVT_ENTRIES; i++) {
     lvt[i] = 0x10000;	// all LVT are masked
   }
 
-  spurious_vector = 0xff;   // software disabled (bit 8)
-
-  // KPL: Register a non-active timer for use when the timer is started.
-  timer_handle = bx_pc_system.register_timer_ticks(this,
-            BX_CPU(0)->local_apic.periodic_smf, 0, 0, 0, "lapic");
+  spurious_vector  = 0xff;   // software disabled (bit 8)
+  software_enabled = 0;
+  focus_disable    = 0;
 }
 
 void bx_local_apic_c::set_id (Bit8u newid)
 {
+  local_apic_index[id] = NULL;
   bx_generic_apic_c::set_id (newid);
   local_apic_index[id] = this;
   sprintf (cpu->name, "CPU apicid=%02x", (Bit32u)id);
@@ -470,11 +492,6 @@ void bx_local_apic_c::set_id (Bit8u newid)
     BX_INFO(( "80%d86", BX_CPU_LEVEL ));
 }
 
-char *bx_local_apic_c::get_name()
-{
-  return cpu->name;
-}
-
 void bx_local_apic_c::set_divide_configuration (Bit32u value)
 {
   BX_ASSERT (value == (value & 0x0b));
@@ -498,7 +515,7 @@ void bx_local_apic_c::write (Bit32u addr, Bit32u *data, unsigned len)
       id = (value>>24) & APIC_ID_MASK;
       break;
     case 0x80: // task priority
-      task_priority = value & 0xff;
+      set_tpr(value & 0xff);
       break;
     case 0xb0: // EOI
       receive_EOI(value);
@@ -512,17 +529,15 @@ void bx_local_apic_c::write (Bit32u addr, Bit32u *data, unsigned len)
       BX_DEBUG (("set destination format to %02x", dest_format));
       break;
     case 0xf0: // spurious interrupt vector
-      // bits 0-3 are hardwired to logical '1
-      spurious_vector = (spurious_vector & 0xff) | (value & 0x30f);
+      write_spurious_interrupt_register(value);
       break;
     case 0x280: // error status reg
       // Here's what the IA-devguide-3 says on p.7-45:
       // The ESR is a read/write register and is reset after being written to
       // by the processor. A write to the ESR must be done just prior to
       // reading the ESR to allow the register to be updated.
-      // This doesn't seem clear.  If the write clears the register, then
-      // wouldn't you always read zero?  Otherwise, what does the write do?
-      error_status = 0;
+      error_status = shadow_error_status;
+      shadow_error_status = 0;
       break;
     case 0x300: // interrupt command reg 0-31
       {
@@ -540,7 +555,7 @@ void bx_local_apic_c::write (Bit32u addr, Bit32u *data, unsigned len)
         bx_bool accepted = deliver (dest, dest_mode, delivery_mode, vector, level, trig_mode);
         if (!accepted) {
           BX_DEBUG(("An IPI wasn't accepted, raise APIC_ERR_TX_ACCEPT_ERR"));
-          error_status |= APIC_ERR_TX_ACCEPT_ERR;
+          shadow_error_status |= APIC_ERR_TX_ACCEPT_ERR;
         }
       }
       break;
@@ -549,45 +564,30 @@ void bx_local_apic_c::write (Bit32u addr, Bit32u *data, unsigned len)
       break;
     case 0x320: // LVT Timer Reg
       lvt[APIC_LVT_TIMER] = value & 0x300ff;
+      if (! software_enabled) lvt[APIC_LVT_TIMER] |= 0x10000;
       break;
     case 0x330: // LVT Thermal Monitor
       lvt[APIC_LVT_THERMAL] = value & 0x107ff;
+      if (! software_enabled) lvt[APIC_LVT_THERMAL] |= 0x10000;
       break;
     case 0x340: // LVT Performance Counter
       lvt[APIC_LVT_PERFORM] = value & 0x107ff;
+      if (! software_enabled) lvt[APIC_LVT_PERFORM] |= 0x10000;
       break;
     case 0x350: // LVT LINT0 Reg
       lvt[APIC_LVT_LINT0] = value & 0x1a7ff;
+      if (! software_enabled) lvt[APIC_LVT_LINT0] |= 0x10000;
       break;
     case 0x360: // LVT Lint1 Reg
       lvt[APIC_LVT_LINT1] = value & 0x1a7ff;
+      if (! software_enabled) lvt[APIC_LVT_LINT1] |= 0x10000;
       break;
     case 0x370: // LVT Error Reg
       lvt[APIC_LVT_ERROR] = value & 0x100ff;
+      if (! software_enabled) lvt[APIC_LVT_ERROR] |= 0x10000;
       break;
     case 0x380: // initial count for timer
-      {
-        // If active before, deactive the current timer before changing it.
-        if (timer_active) {
-          bx_pc_system.deactivate_timer(timer_handle);
-          timer_active = 0;
-        }
-        timer_initial = value;
-        timer_current = 0;
-        if (timer_initial != 0)  // terminate the counting if timer_initial = 0
-        {
-          // This should trigger the counter to start.  If already started,
-          // restart from the new start value.
-          BX_DEBUG(("APIC: Initial Timer Count Register = %u\n", value));
-          timer_current = timer_initial;
-          timer_active = 1;
-          Bit32u timervec = lvt[APIC_LVT_TIMER];
-          bx_bool continuous = (timervec & 0x20000) > 0;
-          ticksInitial = bx_pc_system.time_ticks(); // Take a reading.
-          bx_pc_system.activate_timer_ticks(timer_handle,
-            Bit64u(timer_initial) * Bit64u(timer_divide_factor), continuous);
-        }
-      }
+      set_initial_timer_count(value);
       break;
     case 0x3e0: // timer divide configuration
       // only bits 3, 1, and 0 are writable
@@ -613,12 +613,33 @@ void bx_local_apic_c::write (Bit32u addr, Bit32u *data, unsigned len)
       BX_INFO(("warning: write to read-only APIC register 0x%02x", addr));
       break;
     default:
-      error_status |= APIC_ERR_ILLEGAL_ADDR;
+      shadow_error_status |= APIC_ERR_ILLEGAL_ADDR;
       // but for now I want to know about it in case I missed some.
       BX_PANIC(("APIC register %08x not implemented", addr));
   }
 }
 
+void bx_local_apic_c::write_spurious_interrupt_register(Bit32u value)
+{
+  BX_DEBUG(("write %08x to spurious interrupt register", value));
+
+#ifdef BX_IMPLEMENT_XAPIC
+  spurious_vector = value & 0xff;
+#else
+  // bits 0-3 of the spurious vector hardwired to '1
+  spurious_vector = (value & 0xf0) | 0xf;
+#endif
+
+  software_enabled = (value >> 8) & 1;
+  focus_disable    = (value >> 9) & 1;
+
+  if (! software_enabled) {
+    for (unsigned i=0; i<APIC_LVT_ENTRIES; i++) {
+      lvt[i] |= 0x10000;	// all LVT are masked
+    }
+  }
+}
+
 void bx_local_apic_c::receive_EOI(Bit32u value)
 {
   BX_DEBUG(("%s: Wrote 0x%x to EOI", cpu->name, value));
@@ -626,13 +647,15 @@ void bx_local_apic_c::receive_EOI(Bit32u value)
   if (vec < 0) {
       BX_INFO(("EOI written without any bit in ISR"));
   } else {
-      BX_DEBUG(("%s: local apic received EOI, hopefully for vector 0x%02x", cpu->name, vec));
-      isr[vec] = 0; 
-      service_local_apic ();
+      if (vec != (int) spurious_vector) {
+        BX_DEBUG(("%s: local apic received EOI, hopefully for vector 0x%02x", cpu->name, vec));
+        isr[vec] = 0; 
+        service_local_apic();
+      }
   }
 
   if (bx_dbg.apic)
-      print_status ();
+      print_status();
 }
 
 void bx_local_apic_c::startup_msg (Bit32u vector)
@@ -662,9 +685,9 @@ void bx_local_apic_c::read_aligned (Bit32u addr, Bit32u *data, unsigned len)
   case 0x80: // task priority
     *data = task_priority & 0xff; break;
   case 0x90: // arbitration priority
-    *data = get_apr (); break;
+    *data = get_apr(); break;
   case 0xa0: // processor priority
-    *data = get_ppr (); break;
+    *data = get_ppr(); break;
   case 0xb0: // EOI
     /*
      * Read-modify-write operations should operate without generating
@@ -677,24 +700,54 @@ void bx_local_apic_c::read_aligned (Bit32u addr, Bit32u *data, unsigned len)
   case 0xe0: // destination format
     *data = ((dest_format & 0xf) << 24) | 0x0fffffff; break;
   case 0xf0: // spurious interrupt vector
-    *data = spurious_vector; break;
+    {
+      Bit32u reg = spurious_vector;
+      if (software_enabled) reg |= 0x100;
+      if (focus_disable) reg |= 0x200;
+      *data = reg; 
+    }
+    break;
   case 0x100: case 0x110:
   case 0x120: case 0x130:
   case 0x140: case 0x150:
   case 0x160: case 0x170:
-    *data = isr[addr2-0x100];
+    {
+      unsigned index = (addr2 - 0x100) >> 2;
+      Bit32u value = 0, mask = 1;
+      for (int i=0;i<32;i++) {
+        if (isr[index+i]) value |= mask;
+        mask <<= 1;
+      }
+      *data = value;
+    }
     break;
   case 0x180: case 0x190:
   case 0x1a0: case 0x1b0:
   case 0x1c0: case 0x1d0:
   case 0x1e0: case 0x1f0:
-    *data = tmr[addr2-0x180];
+    {
+      unsigned index = (addr2 - 0x180) >> 2;
+      Bit32u value = 0, mask = 1;
+      for (int i=0;i<32;i++) {
+        if (tmr[index+i]) value |= mask;
+        mask <<= 1;
+      }
+      *data = value;
+    }
     break;
   case 0x200: case 0x210:
   case 0x220: case 0x230:
   case 0x240: case 0x250:
   case 0x260: case 0x270:
-    *data = irr[addr2-0x200];
+    {
+      unsigned index = (addr2 - 0x200) >> 2;
+      Bit32u value = 0, mask = 1;
+      for (int i=0;i<32;i++) {
+        if (irr[index+i]) value |= mask;
+        mask <<= 1;
+      }
+      *data = value;
+    }
     break;
   case 0x280: // error status reg
     *data = error_status; break;
@@ -749,23 +802,27 @@ int bx_local_apic_c::highest_priority_int (Bit8u *array)
   return -1;
 }
 
-void bx_local_apic_c::service_local_apic ()
+void bx_local_apic_c::service_local_apic(void)
 {
   if (bx_dbg.apic) {
     BX_INFO(("service_local_apic()"));
-    print_status ();
+    print_status();
   }
   if (INTR) return;  // INTR already up; do nothing
   // find first interrupt in irr.
-  int first_irr = highest_priority_int (irr);
-  int first_isr = highest_priority_int (isr);
+  int first_irr = highest_priority_int(irr);
+  int first_isr = highest_priority_int(isr);
   if (first_irr < 0) return;   // no interrupts, leave INTR=0
   if (first_isr >= 0 && first_irr >= first_isr) {
     BX_DEBUG(("local apic (%s): not delivering int%02x because int%02x is in service", cpu->name, first_irr, first_isr));
-    // error_status |= APIC_ERR_TX_ACCEPT_ERR;
+    // shadow_error_status |= APIC_ERR_TX_ACCEPT_ERR;
     return;
   }
-  // interrupt has appeared in irr.  raise INTR.  When the CPU
+  if ((first_irr & 0xf0) <= (task_priority & 0xf0)) {
+    BX_DEBUG(("local apic (%s): not delivering int%02X because task_priority is %X", cpu->name, first_irr, task_priority));
+    return;
+  }
+  // interrupt has appeared in irr. Raise INTR. When the CPU
   // acknowledges, we will run highest_priority_int again and
   // return it.
   BX_DEBUG(("service_local_apic(): setting INTR=1 for vector 0x%02x", first_irr));
@@ -780,13 +837,12 @@ void bx_local_apic_c::trigger_irq (unsigned vector, unsigned from, unsigned trig
   BX_ASSERT(from == id);
   
   if (vector > BX_APIC_LAST_VECTOR || vector < BX_APIC_FIRST_VECTOR) {
-    error_status |= APIC_ERR_RX_ILLEGAL_VEC;
+    shadow_error_status |= APIC_ERR_RX_ILLEGAL_VEC;
     BX_INFO(("bogus vector %#x, ignoring ...", vector));
     return;
   }
 
-  if (bx_dbg.apic)
-    BX_INFO(("triggered vector %#02x", vector));
+  BX_DEBUG(("triggered vector %#02x", vector));
 
   if (bypass_irr_isr) {
     bypass_irr_isr = 0;
@@ -801,7 +857,7 @@ void bx_local_apic_c::trigger_irq (unsigned vector, unsigned from, unsigned trig
 service_vector:
   irr[vector] = 1;
   tmr[vector] = trigger_mode;	// set for level triggered
-  service_local_apic ();
+  service_local_apic();
 }
 
 void bx_local_apic_c::untrigger_irq (unsigned vector, unsigned from, unsigned trigger_mode)
@@ -811,18 +867,20 @@ void bx_local_apic_c::untrigger_irq (unsigned vector, unsigned from, unsigned tr
   // acknowledged the interrupt, it will never be serviced.
   BX_ASSERT (irr[vector] == 1);
   irr[vector] = 0;
-  if (bx_dbg.apic) print_status ();
+  if (bx_dbg.apic) print_status();
 }
 
-Bit8u bx_local_apic_c::acknowledge_int ()
+Bit8u bx_local_apic_c::acknowledge_int(void)
 {
   // CPU calls this when it is ready to service one interrupt
   if (!INTR)
     BX_PANIC(("%s: acknowledged an interrupt, but INTR=0", cpu->name));
   BX_ASSERT (INTR);
-  int vector = highest_priority_int (irr);
+  int vector = highest_priority_int(irr);
+  if (vector < 0) goto spurious;
+  if ((vector & 0xf0) <= get_ppr()) goto spurious;
   if (irr[vector] != 1) {
-    BX_ERROR(("IRR was not 1! irr[%d]=%#x", vector, irr[vector]));
+    BX_PANIC(("IRR was not 1! irr[%d]=%#x", vector, irr[vector]));
     irr[vector]=1;
   }
   BX_ASSERT (irr[vector] == 1);
@@ -833,15 +891,21 @@ Bit8u bx_local_apic_c::acknowledge_int ()
   isr[vector] = 1;
   if (bx_dbg.apic) {
     BX_INFO(("Status after setting isr:"));
-    print_status ();
+    print_status();
   }
   INTR = 0;
   cpu->async_event = 1;
-  service_local_apic ();  // will set INTR again if another is ready
+  service_local_apic();  // will set INTR again if another is ready
   return vector;
+
+spurious:
+  INTR = 0;
+  cpu->async_event = 1;
+  return spurious_vector;
+
 }
 
-void bx_local_apic_c::print_status ()
+void bx_local_apic_c::print_status (void)
 {
   BX_INFO(("%s local apic: status is {:", cpu->name));
   for (int vec=0; vec<BX_LOCAL_APIC_MAX_INTS; vec++) {
@@ -863,7 +927,7 @@ bx_bool bx_local_apic_c::match_logical_addr (Bit8u address)
   return match;
 }
 
-Bit8u bx_local_apic_c::get_ppr ()
+Bit8u bx_local_apic_c::get_ppr (void)
 {
   int ppr = highest_priority_int (isr);
 
@@ -875,57 +939,63 @@ Bit8u bx_local_apic_c::get_ppr ()
   return ppr;
 }
 
-Bit8u bx_local_apic_c::get_tpr ()
+Bit8u bx_local_apic_c::get_tpr (void)
 {
   return task_priority;
 }
 
 void  bx_local_apic_c::set_tpr (Bit8u priority)
 {
-  task_priority = priority;
+  if (priority < task_priority) {
+    task_priority = priority;
+    service_local_apic();
+  } else {
+    task_priority = priority;
+  }
 }
 
-Bit8u bx_local_apic_c::get_apr ()
+Bit8u bx_local_apic_c::get_apr(void)
 {
-  return arb_id;
-}
-
-Bit8u bx_local_apic_c::get_apr_lowpri()
-{
-  Bit32u tpr = (task_priority >> 4) & 0xf;
+  Bit32u tpr  = (task_priority >> 4) & 0xf;
   Bit32u isrv = (highest_priority_int(isr) >> 4) & 0xf;
   Bit32u irrv = (highest_priority_int(irr) >> 4) & 0xf;
+  Bit8u  apr;
+
+  if (isrv < 0) isrv = 0;
+  if (irrv < 0) irrv = 0;
 
   if ((tpr >= irrv) && (tpr > isrv)) {
-    arb_id = task_priority & 0xff;
+    apr = task_priority & 0xff;
   }
   else {
-    arb_id = ((tpr & isrv) > irrv) ? (tpr & isrv) : irrv;
-    arb_id <<= 4;
+    apr = ((tpr & isrv) > irrv) ? (tpr & isrv) : irrv;
+    apr <<= 4;
   }
 
-  BX_INFO(("apr = %d\n", arb_id));
-  return (Bit8u)arb_id;
+  BX_DEBUG(("apr = %d\n", apr));
+
+  return (Bit8u) apr;
 }
 
-bx_bool bx_local_apic_c::is_focus(Bit32u vector)
+bx_bool bx_local_apic_c::is_focus(Bit8u vector)
 {
+  if (focus_disable) return 0;
   return (irr[vector] || isr[vector]) ? 1 : 0;
 }
 
 void bx_local_apic_c::adjust_arb_id(int winning_id)
 {
-  int __apr, __win_apr;
+  int __arb_id, __win_arb_id;
   // adjust arbitration priorities
-  for (int i = 0; i < BX_LOCAL_APIC_NUM; i++) {
+  for (int i = 0; i < BX_NUM_LOCAL_APICS; i++) {
     if (i != winning_id) {
-      __apr = local_apic_index[i]->get_apr();
-      if (__apr == 15) {
-        __win_apr = local_apic_index[winning_id]->get_apr();
-        local_apic_index[i]->set_arb_id(__win_apr+1);
+      __arb_id = local_apic_index[i]->get_arb_id();
+      if (__arb_id == APIC_MAX_ID) {
+        __win_arb_id = local_apic_index[winning_id]->get_arb_id();
+        local_apic_index[i]->set_arb_id(__win_arb_id+1);
       } 
       else
-        local_apic_index[i]->set_arb_id(__apr+1);
+        local_apic_index[i]->set_arb_id(__arb_id+1);
     } else // the winner drops to lowest
       local_apic_index[winning_id]->set_arb_id(0);
   }
@@ -970,4 +1040,30 @@ void bx_local_apic_c::periodic(void)
   }
 }
 
+void bx_local_apic_c::set_initial_timer_count(Bit32u value)
+{
+  // If active before, deactive the current timer before changing it.
+  if (timer_active) {
+    bx_pc_system.deactivate_timer(timer_handle);
+    timer_active = 0;
+  }
+
+  timer_initial = value;
+  timer_current = 0;
+
+  if (timer_initial != 0)  // terminate the counting if timer_initial = 0
+  {
+    // This should trigger the counter to start.  If already started,
+    // restart from the new start value.
+    BX_DEBUG(("APIC: Initial Timer Count Register = %u\n", value));
+    timer_current = timer_initial;
+    timer_active = 1;
+    Bit32u timervec = lvt[APIC_LVT_TIMER];
+    bx_bool continuous = (timervec & 0x20000) > 0;
+    ticksInitial = bx_pc_system.time_ticks(); // Take a reading.
+    bx_pc_system.activate_timer_ticks(timer_handle,
+            Bit64u(timer_initial) * Bit64u(timer_divide_factor), continuous);
+  }
+}
+
 #endif /* if BX_SUPPORT_APIC */
