@@ -94,8 +94,7 @@ void bx_external_debugger(BX_CPU_C *cpu)
      regs.fsbase = cpu->sregs[BX_SEG_REG_FS].cache.u.segment.base;
      regs.gsbase = cpu->sregs[BX_SEG_REG_GS].cache.u.segment.base;
 #if BX_SUPPORT_X86_64
-    regs.efer = (cpu->msr.sce << 0)
-               | (cpu->msr.lme << 8) | (cpu->msr.lma << 10);
+    regs.efer = cpu->get_EFER();
 #else
     regs.efer = 0;
 #endif
