@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: init.cc,v 1.70.2.1 2005/07/07 08:00:41 vruppert Exp $
+// $Id: init.cc,v 1.76 2005/12/12 19:54:48 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -29,7 +29,7 @@
 #include "bochs.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-BX_CPU_C::BX_CPU_C(): bx_cpuid(0)
+BX_CPU_C::BX_CPU_C(unsigned id): bx_cpuid(id)
 #if BX_SUPPORT_APIC
    ,local_apic (this)
 #endif
@@ -158,9 +158,8 @@ cpu_param_handler (bx_param_c *param, int set, Bit64s val)
 
 #endif
 
-void BX_CPU_C::init(BX_MEM_C *addrspace)
+void BX_CPU_C::initialize(BX_MEM_C *addrspace)
 {
-  BX_DEBUG(( "Init $Id: init.cc,v 1.70.2.1 2005/07/07 08:00:41 vruppert Exp $"));
   // BX_CPU_C constructor
   BX_CPU_THIS_PTR set_INTR (0);
 #if BX_SUPPORT_APIC
@@ -283,7 +282,7 @@ void BX_CPU_C::init(BX_MEM_C *addrspace)
 // <TAG-INIT-CPU-END>
 
   mem = addrspace;
-  sprintf (name, "CPU %p", this);
+  sprintf (name, "CPU %d", which_cpu());
 
 #if BX_WITH_WX
   static bx_bool first_time = 1;
@@ -505,8 +504,8 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.valid    = 1;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.p        = 1;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.dpl      = 0;
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.segment  = 1; /* data/code segment */
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.type     = 3; /* read/write access */
+  BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.segment  = 1;  /* data/code segment */
+  BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.type     = 11; /* executable/readable/access code segment */
 
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.executable   = 1; /* data/stack segment */
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.c_ed         = 0; /* normal expand up */
@@ -519,6 +518,9 @@ void BX_CPU_C::reset(unsigned source)
 #if BX_CPU_LEVEL >= 3
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.g   = 0; /* byte granular */
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.d_b = 0; /* 16bit default size */
+#if BX_SUPPORT_X86_64
+  BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.l   = 0; /* 16bit default size */
+#endif
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.avl = 0;
 #endif
 
@@ -533,7 +535,7 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.p        = 1;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.dpl      = 0;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.segment  = 1; /* data/code segment */
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.type     = 3; /* read/write access */
+  BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.type     = 3; /* read/write/access */
 
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.executable   = 0; /* data/stack segment */
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.c_ed         = 0; /* normal expand up */
@@ -546,6 +548,9 @@ void BX_CPU_C::reset(unsigned source)
 #if BX_CPU_LEVEL >= 3
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.g   = 0; /* byte granular */
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.d_b = 0; /* 16bit default size */
+#if BX_SUPPORT_X86_64
+  BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.l   = 0; /* 16bit default size */
+#endif
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.avl = 0;
 #endif
 
@@ -560,7 +565,7 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.p        = 1;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.dpl      = 0;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.segment  = 1; /* data/code segment */
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.type     = 3; /* read/write access */
+  BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.type     = 3; /* read/write/access */
 
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.executable   = 0; /* data/stack segment */
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.c_ed         = 0; /* normal expand up */
@@ -573,6 +578,9 @@ void BX_CPU_C::reset(unsigned source)
 #if BX_CPU_LEVEL >= 3
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.g   = 0; /* byte granular */
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.d_b = 0; /* 16bit default size */
+#if BX_SUPPORT_X86_64
+  BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.l   = 0; /* 16bit default size */
+#endif
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].cache.u.segment.avl = 0;
 #endif
 
@@ -587,7 +595,7 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.p        = 1;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.dpl      = 0;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.segment  = 1; /* data/code segment */
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.type     = 3; /* read/write access */
+  BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.type     = 3; /* read/write/access */
 
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.executable   = 0; /* data/stack segment */
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.c_ed         = 0; /* normal expand up */
@@ -600,6 +608,9 @@ void BX_CPU_C::reset(unsigned source)
 #if BX_CPU_LEVEL >= 3
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.g   = 0; /* byte granular */
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.d_b = 0; /* 16bit default size */
+#if BX_SUPPORT_X86_64
+  BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.l   = 0; /* 16bit default size */
+#endif
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].cache.u.segment.avl = 0;
 #endif
 
@@ -614,7 +625,7 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.p       = 1;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.dpl     = 0;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.segment = 1; /* data/code segment */
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.type    = 3; /* read/write access */
+  BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.type    = 3; /* read/write/access */
 
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.executable   = 0; /* data/stack segment */
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.c_ed         = 0; /* normal expand up */
@@ -625,6 +636,9 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.limit_scaled =     0xFFFF;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.g   = 0; /* byte granular */
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.d_b = 0; /* 16bit default size */
+#if BX_SUPPORT_X86_64
+  BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.l   = 0; /* 16bit default size */
+#endif
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].cache.u.segment.avl = 0;
 
   /* GS and descriptor cache */
@@ -637,7 +651,7 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.p        = 1;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.dpl      = 0;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.segment  = 1; /* data/code segment */
-  BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.type     = 3; /* read/write access */
+  BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.type     = 3; /* read/write/access */
 
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.executable   = 0; /* data/stack segment */
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.c_ed         = 0; /* normal expand up */
@@ -648,17 +662,20 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.limit_scaled =     0xFFFF;
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.g   = 0; /* byte granular */
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.d_b = 0; /* 16bit default size */
+#if BX_SUPPORT_X86_64
+  BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.l   = 0; /* 16bit default size */
+#endif
   BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].cache.u.segment.avl = 0;
 #endif
 
 #if BX_CPU_LEVEL >= 2
   /* GDTR (Global Descriptor Table Register) */
-  BX_CPU_THIS_PTR gdtr.base         = 0x00000000;
-  BX_CPU_THIS_PTR gdtr.limit        =     0xFFFF;
+  BX_CPU_THIS_PTR gdtr.base  = 0x00000000;
+  BX_CPU_THIS_PTR gdtr.limit =     0xFFFF;
 
   /* IDTR (Interrupt Descriptor Table Register) */
-  BX_CPU_THIS_PTR idtr.base         = 0x00000000;
-  BX_CPU_THIS_PTR idtr.limit        =     0xFFFF; /* always byte granular */
+  BX_CPU_THIS_PTR idtr.base  = 0x00000000;
+  BX_CPU_THIS_PTR idtr.limit =     0xFFFF; /* always byte granular */
 
   /* LDTR (Local Descriptor Table Register) */
   BX_CPU_THIS_PTR ldtr.selector.value = 0x0000;
@@ -666,8 +683,8 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR ldtr.selector.ti    = 0;
   BX_CPU_THIS_PTR ldtr.selector.rpl   = 0;
 
-  BX_CPU_THIS_PTR ldtr.cache.valid    = 0; /* not valid */
-  BX_CPU_THIS_PTR ldtr.cache.p        = 0; /* not present */
+  BX_CPU_THIS_PTR ldtr.cache.valid    = 1; /* valid */
+  BX_CPU_THIS_PTR ldtr.cache.p        = 1; /* present */
   BX_CPU_THIS_PTR ldtr.cache.dpl      = 0; /* field not used */
   BX_CPU_THIS_PTR ldtr.cache.segment  = 0; /* system segment */
   BX_CPU_THIS_PTR ldtr.cache.type     = 2; /* LDT descriptor */
@@ -681,13 +698,13 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR tr.selector.ti    = 0;
   BX_CPU_THIS_PTR tr.selector.rpl   = 0;
 
-  BX_CPU_THIS_PTR tr.cache.valid    = 0;
-  BX_CPU_THIS_PTR tr.cache.p        = 0;
+  BX_CPU_THIS_PTR tr.cache.valid    = 1; /* valid */
+  BX_CPU_THIS_PTR tr.cache.p        = 1; /* present */
   BX_CPU_THIS_PTR tr.cache.dpl      = 0; /* field not used */
-  BX_CPU_THIS_PTR tr.cache.segment  = 0;
-  BX_CPU_THIS_PTR tr.cache.type     = 0; /* invalid */
-  BX_CPU_THIS_PTR tr.cache.u.tss286.base     = 0x00000000; /* undefined */
-  BX_CPU_THIS_PTR tr.cache.u.tss286.limit    =     0x0000; /* undefined */
+  BX_CPU_THIS_PTR tr.cache.segment  = 0; /* system segment */
+  BX_CPU_THIS_PTR tr.cache.type     = 3; /* busy 16-bit TSS */
+  BX_CPU_THIS_PTR tr.cache.u.tss286.base  = 0x00000000;
+  BX_CPU_THIS_PTR tr.cache.u.tss286.limit =     0xFFFF;
 #endif
 
   // DR0 - DR7 (Debug Registers)
@@ -798,10 +815,11 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR magic_break = 0;
 #endif
   BX_CPU_THIS_PTR stop_reason = STOP_NO_REASON;
-  BX_CPU_THIS_PTR trace = 0;
   BX_CPU_THIS_PTR trace_reg = 0;
 #endif
 
+  BX_CPU_THIS_PTR trace = 0;
+
   // Reset the Floating Point Unit
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR the_i387.reset(); // unchanged on #INIT
