@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: sse.cc,v 1.32 2005/05/12 18:07:43 sshwarts Exp $
+// $Id: sse.cc,v 1.34 2005/09/23 16:45:41 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
@@ -365,7 +365,7 @@ void BX_CPU_C::PINSRW_VdqEdIb(bxInstruction_c *i)
 }
 
 /* 66 0F C5 */
-void BX_CPU_C::PEXTRW_VdqEdIb(bxInstruction_c *i)
+void BX_CPU_C::PEXTRW_GdVRdqIb(bxInstruction_c *i)
 {
 #if BX_SUPPORT_SSE >= 2
   BX_CPU_THIS_PTR prepareSSE();
@@ -374,9 +374,9 @@ void BX_CPU_C::PEXTRW_VdqEdIb(bxInstruction_c *i)
   Bit8u count = i->Ib() & 0x7;
   Bit32u result = (Bit32u) op.xmm16u(count);
 
-  BX_WRITE_32BIT_REG(i->nnn(), result);
+  BX_WRITE_32BIT_REGZ(i->nnn(), result);
 #else
-  BX_INFO(("PEXTRW_VdqEdIb: required SSE2, use --enable-sse option"));
+  BX_INFO(("PEXTRW_GdVRdqIb: required SSE2, use --enable-sse option"));
   UndefinedOpcode(i);
 #endif
 }
@@ -601,7 +601,7 @@ void BX_CPU_C::PMOVMSKB_GdVRdq(bxInstruction_c *i)
   if(op.xmmubyte(0xF) & 0x80) result |= 0x8000; 
 
   /* now write result back to destination */
-  BX_WRITE_32BIT_REG(i->nnn(), result);
+  BX_WRITE_32BIT_REGZ(i->nnn(), result);
   
 #else
   BX_INFO(("PMOVMSKB_GdVRdq: required SSE2, use --enable-sse option"));
