@@ -1,11 +1,9 @@
 /////////////////////////////////////////////////////////////////////////
-//  Copyright (C) 2004  MandrakeSoft S.A.
+// $Id: fpu_load_store.cc,v 1.8 2005/05/12 18:07:46 sshwarts Exp $
+/////////////////////////////////////////////////////////////////////////
 //
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//   Copyright (c) 2003 Stanislav Shwartsman
+//          Written by Stanislav Shwartsman <stl at fidonet.org.il>
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -20,9 +18,9 @@
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #define LOG_THIS BX_CPU_THIS_PTR
@@ -246,7 +244,7 @@ void BX_CPU_C::FBLD_PACKED_BCD(bxInstruction_c *i)
   Bit64s scale = 1; 
   Bit64s val64 = 0;
 
-  for (int i = 0; i < 16; i++)
+  for (int n = 0; n < 16; n++)
   {
     val64 += (lo8 & 0x0F) * scale;
     lo8 >>= 4;
@@ -550,9 +548,9 @@ void BX_CPU_C::FBSTP_PACKED_BCD(bxInstruction_c *i)
            save_val /= 10;
         }
 
-        save_reg_hi += (save_val % 10);
+        save_reg_hi += (Bit16u)(save_val % 10);
         save_val /= 10;
-        save_reg_hi += (save_val % 10) << 4;
+        save_reg_hi += (Bit16u)(save_val % 10) << 4;
     }
 
     /* check for fpu arithmetic exceptions */
