@@ -1,16 +1,22 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
+#ifdef __MINGW32__
+#include <winsock2.h>
+#define SIGTRAP 5
+#else
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
 #include <signal.h>
 #include <netdb.h>
+#endif
 
 #define NEED_CPU_REG_SHORTCUTS 1
 
 #include "bochs.h"
+#include "iodev/iodev.h"
 
 #define LOG_THIS genlog->
 #define IFDBG(x) x
@@ -164,16 +170,43 @@ char* mem2hex(char* mem, char* buf, int count)
    return(buf);
 }
 
+int hexdigit(char c)
+{
+  if (isdigit(c))
+    return c - '0';
+  else if (isupper(c))
+    return c - 'A' + 10;
+  else
+    return c - 'a' + 10;
+}
+
+Bit64u read_little_endian_hex(char *&buf)
+{
+  int byte;
+  Bit64u ret = 0;
+  int n = 0;
+  while (isxdigit(*buf)) {
+    byte = hexdigit(*buf++);
+    if (isxdigit(*buf))
+      byte = (byte << 4) | hexdigit(*buf++);
+    ret |= (unsigned long long)byte << (n*8);
+    ++n;
+  }
+  return ret;
+}
+
 static int continue_thread = -1;
 static int other_thread = 0;
 
+#if !BX_SUPPORT_X86_64
 #define NUMREGS (16)
 #define NUMREGSBYTES (NUMREGS * 4)
 static int registers[NUMREGS];
+#endif
 
 #define MAX_BREAKPOINTS (255)
-static int breakpoints[MAX_BREAKPOINTS] = {0,};
-static int nr_breakpoints = 0;
+static unsigned int breakpoints[MAX_BREAKPOINTS] = {0,};
+static unsigned int nr_breakpoints = 0;
 
 static int stub_trace_flag = 0;
 
@@ -187,14 +220,16 @@ int bx_gdbstub_check(unsigned int eip)
    unsigned char ch;
    long arg;
    int r;
+#if defined(__CYGWIN__) || defined(__MINGW32__)
    fd_set fds;
    struct timeval tv = {0, 0};
+#endif
 
    instr_count++;
    
    if ((instr_count % 500) == 0)
      {
-#ifndef __CYGWIN__
+#if !defined(__CYGWIN__) && !defined(__MINGW32__)
        arg = fcntl(socket_fd, F_GETFL);
        fcntl(socket_fd, F_SETFL, arg | O_NONBLOCK);
        r = recv(socket_fd, &ch, 1, 0);
@@ -205,7 +240,7 @@ int bx_gdbstub_check(unsigned int eip)
         r = select(socket_fd + 1, &fds, NULL, NULL, &tv);
        if (r == 1)
          {
-           r = recv(socket_fd, &ch, 1, 0);
+           r = recv(socket_fd, (char *)&ch, 1, 0);
          }
 #endif   
        if (r == 1)
@@ -236,7 +271,7 @@ int bx_gdbstub_check(unsigned int eip)
    return(GDBSTUB_STOP_NO_REASON);
 }
 
-static int remove_breakpoint(int addr, int len)
+static int remove_breakpoint(unsigned int addr, int len)
 {
    unsigned int i;
    
@@ -257,13 +292,13 @@ static int remove_breakpoint(int addr, int len)
    return(0);
 }
 
-static void insert_breakpoint(int addr)
+static void insert_breakpoint(unsigned int addr)
 {
    unsigned int i;
    
    BX_INFO (("setting breakpoint at %x", addr));
    
-   for (i = 0; i < MAX_BREAKPOINTS; i++)
+   for (i = 0; i < (unsigned)MAX_BREAKPOINTS; i++)
      {
        if (breakpoints[i] == 0)
          {
@@ -278,6 +313,33 @@ static void insert_breakpoint(int addr)
    BX_INFO (("No slot for breakpoint"));
 }
 
+static void do_pc_breakpoint(int insert, unsigned long long addr, int len)
+{
+  for (int i = 0; i < len; ++i)
+    if (insert)
+      insert_breakpoint(addr+i);
+    else
+      remove_breakpoint(addr+i, 1);
+}
+
+static void do_breakpoint(int insert, char* buffer)
+{
+  char* ebuf;
+  unsigned long type = strtoul(buffer, &ebuf, 16);
+  unsigned long long addr = strtoull(ebuf+1, &ebuf, 16);
+  unsigned long len = strtoul(ebuf+1, &ebuf, 16);
+  switch (type) {
+  case 0:
+  case 1:
+    do_pc_breakpoint(insert, addr, len);
+    put_reply("OK");
+    break;
+  default:
+    put_reply("");
+    break;
+  }
+}
+
 static void write_signal(char* buf, int signal)
 {
    buf[0] = hexchars[signal >> 4];
@@ -285,7 +347,7 @@ static void write_signal(char* buf, int signal)
    buf[2] = 0;
 }
 
-static int access_linear(Bit32u laddress,
+static int access_linear(Bit64u laddress,
                         unsigned len,
                         unsigned int rw,
                         Bit8u* data)
@@ -311,7 +373,7 @@ static int access_linear(Bit32u laddress,
        return(valid);
      }
    
-   BX_CPU(0)->dbg_xlate_linear2phy((Bit32u)laddress, 
+   BX_CPU(0)->dbg_xlate_linear2phy(laddress, 
                                        (Bit32u*)&phys, 
                                        (bx_bool*)&valid);
    if (!valid)
@@ -367,7 +429,13 @@ static void debug_loop(void)
                    }
                  
                  stub_trace_flag = 0;
+                 bx_cpu.ispanic = 0;
                  bx_cpu.cpu_loop(-1);              
+                 if (bx_cpu.ispanic)
+                 {
+                    last_stop_reason = GDBSTUB_EXECUTION_BREAKPOINT;
+                 }
+
                  DEV_vga_refresh();
                  
                  if (buffer[1] != 0)
@@ -418,12 +486,12 @@ static void debug_loop(void)
             
           case 'M':
               {
-                 int addr;
+                 Bit64u addr;
                  int len;
                  unsigned char mem[255];
                  char* ebuf;
                  
-                 addr = strtoul(&buffer[1], &ebuf, 16);
+                 addr = strtoull(&buffer[1], &ebuf, 16);
                  len = strtoul(ebuf + 1, &ebuf, 16);
                  hex2mem(ebuf + 1, mem, len);          
                  
@@ -455,11 +523,11 @@ static void debug_loop(void)
             
           case 'm':
               {
-                 int addr;
+                 Bit64u addr;
                  int len;
                  char* ebuf;
                  
-                 addr = strtoul(&buffer[1], &ebuf, 16);
+                 addr = strtoull(&buffer[1], &ebuf, 16);
                  len = strtoul(ebuf + 1, NULL, 16);
                  BX_INFO (("addr %x len %x", addr, len));
                  
@@ -474,14 +542,15 @@ static void debug_loop(void)
           case 'P':
               {
                  int reg;
-                 int value;
+                 unsigned long long value;
                  char* ebuf;
                  
                  reg = strtoul(&buffer[1], &ebuf, 16);
-                 value = ntohl(strtoul(ebuf + 1, &ebuf, 16));
-                 
-                 BX_INFO (("reg %d set to %x", reg, value));
+		 ++ebuf;
+                 value = read_little_endian_hex(ebuf);
                  
+                 BX_INFO (("reg %d set to %llx", reg, value));
+#if !BX_SUPPORT_X86_64                 
                  switch (reg)
                    {
                     case 1:
@@ -520,13 +589,89 @@ static void debug_loop(void)
                     default:
                       break;
                    }
-                 
+#else
+                 switch (reg)
+                   {
+                    case 0:
+                      RAX = value;
+                      break;
+                      
+                    case 1:
+                      RBX = value;
+                      break;
+                      
+                    case 2:
+                      RCX = value;
+                      break;
+                      
+                    case 3:
+                      RDX = value;
+                      break;
+                      
+                    case 4:
+                      RSI = value;
+                      break;
+                      
+                    case 5:
+                      RDI = value;
+                      break;
+                      
+                    case 6:
+                      ESP = value;
+                      break;
+                      
+                    case 7:
+		      RBP = value;
+		      break;
+
+                    case 8:
+		      R8 = value;
+		      break;
+
+                    case 9:
+		      R9 = value;
+		      break;
+
+                    case 10:
+		      R10 = value;
+		      break;
+
+                    case 11:
+		      R11 = value;
+		      break;
+
+                    case 12:
+		      R12 = value;
+		      break;
+
+                    case 13:
+		      R13 = value;
+		      break;
+
+                    case 14:
+		      R15 = value;
+		      break;
+
+                    case 15:
+		      R15 = value;
+		      break;
+
+		    case 16:
+                      RIP = value;
+                      BX_CPU_THIS_PTR invalidate_prefetch_q();
+                      break;
+                      
+                    default:
+                      break;
+                   }
+#endif                 
                  put_reply("OK");
                  
                  break;
               }
             
           case 'g':
+#if !BX_SUPPORT_X86_64
             registers[0] = EAX;
             registers[1] = ECX;
             registers[2] = EDX;
@@ -557,6 +702,44 @@ static void debug_loop(void)
             registers[15] = 
               BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].selector.value;
             mem2hex((char *)registers, obuf, NUMREGSBYTES);
+#else
+#define PUTREG(buf, val, len) do { \
+         Bit64u u = (val); \
+         (buf) = mem2hex((char*)&u, (buf), (len)); \
+      } while (0)
+            char* buf;
+	    buf = obuf;
+            PUTREG(buf, RAX, 8);
+            PUTREG(buf, RBX, 8);
+            PUTREG(buf, RCX, 8);
+            PUTREG(buf, RDX, 8);
+            PUTREG(buf, RSI, 8);
+            PUTREG(buf, RDI, 8);
+            PUTREG(buf, RBP, 8);
+            PUTREG(buf, RSP, 8);
+            PUTREG(buf, R8,  8);
+            PUTREG(buf, R9,  8);
+            PUTREG(buf, R10, 8);
+            PUTREG(buf, R11, 8);
+            PUTREG(buf, R12, 8);
+            PUTREG(buf, R13, 8);
+            PUTREG(buf, R14, 8);
+            PUTREG(buf, R15, 8);
+	    Bit64u rip;
+	    rip = RIP;
+            if (last_stop_reason == GDBSTUB_EXECUTION_BREAKPOINT)
+              {
+		++rip;
+              }
+            PUTREG(buf, rip, 8);
+	    PUTREG(buf, BX_CPU_THIS_PTR read_eflags(), 4);
+            PUTREG(buf, BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, 4);
+            PUTREG(buf, BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].selector.value, 4);
+            PUTREG(buf, BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].selector.value, 4);
+            PUTREG(buf, BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].selector.value, 4);
+            PUTREG(buf, BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].selector.value, 4);
+            PUTREG(buf, BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].selector.value, 4);
+#endif
             put_reply(obuf);
             break;
             
@@ -602,7 +785,13 @@ static void debug_loop(void)
                  put_reply("ENN");
               }          
             break;
-            
+
+	  case 'Z':
+	    do_breakpoint(1, buffer+1);
+	    break;
+	  case 'z':
+	    do_breakpoint(0, buffer+1);
+	    break;
           case 'k':
             BX_PANIC (("Debugger asked us to quit\n"));
             break;
@@ -631,7 +820,11 @@ static void wait_for_connect(int portn)
    
    /* Allow rapid reuse of this port */
    opt = 1;
+#if __MINGW32__
+   r = setsockopt(listen_socket_fd, SOL_SOCKET, SO_REUSEADDR, (const char *)&opt, sizeof(opt));
+#else
    r = setsockopt(listen_socket_fd, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));
+#endif
    if (r == -1)
      {
        BX_INFO (("setsockopt(SO_REUSEADDR) failed\n"));
@@ -676,7 +869,11 @@ static void wait_for_connect(int portn)
 
    /* Disable Nagle - allow small packets to be sent without delay. */
    opt = 1;
+#ifdef __MINGW32__
+   r = setsockopt (socket_fd, protoent->p_proto, TCP_NODELAY, (const char *)&opt, sizeof(opt));
+#else
    r = setsockopt (socket_fd, protoent->p_proto, TCP_NODELAY, &opt, sizeof(opt));
+#endif
    if (r == -1)
      {
        BX_INFO (("setsockopt(TCP_NODELAY) failed\n"));
@@ -687,6 +884,11 @@ void bx_gdbstub_init(int argc, char* argv[])
 {
    int portn = bx_options.gdbstub.port;
 
+#ifdef __MINGW32__
+   WSADATA wsaData;
+   WSAStartup(2, &wsaData);
+#endif
+
    bx_init_hardware();
 
    /* Wait for connect */
