@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: gui.cc,v 1.73 2003/12/18 20:04:48 vruppert Exp $
+// $Id: gui.cc,v 1.87 2005/11/12 16:09:55 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -27,6 +27,7 @@
 
 #include <signal.h>
 #include "bochs.h"
+#include "iodev.h"
 #include "gui/bitmaps/floppya.h"
 #include "gui/bitmaps/floppyb.h"
 #include "gui/bitmaps/mouse.h"
@@ -47,19 +48,76 @@ bx_gui_c *bx_gui = NULL;
 #define BX_GUI_THIS bx_gui->
 #define LOG_THIS BX_GUI_THIS
 
+#define BX_KEY_UNKNOWN 0x7fffffff
+#define N_USER_KEYS 34
+
+typedef struct {
+  char *key;
+  Bit32u symbol;
+} user_key_t;
+
+static user_key_t user_keys[N_USER_KEYS] =
+{
+  { "f1",    BX_KEY_F1 },
+  { "f2",    BX_KEY_F2 },
+  { "f3",    BX_KEY_F3 },
+  { "f4",    BX_KEY_F4 },
+  { "f5",    BX_KEY_F5 },
+  { "f6",    BX_KEY_F6 },
+  { "f7",    BX_KEY_F7 },
+  { "f8",    BX_KEY_F8 },
+  { "f9",    BX_KEY_F9 },
+  { "f10",   BX_KEY_F10 },
+  { "f11",   BX_KEY_F11 },
+  { "f12",   BX_KEY_F12 },
+  { "alt",   BX_KEY_ALT_L },
+  { "bksp",  BX_KEY_BACKSPACE },
+  { "ctrl",  BX_KEY_CTRL_L },
+  { "del",   BX_KEY_DELETE },
+  { "down",  BX_KEY_DOWN },
+  { "end",   BX_KEY_END },
+  { "enter", BX_KEY_ENTER },
+  { "esc",   BX_KEY_ESC },
+  { "home",  BX_KEY_HOME },
+  { "ins",   BX_KEY_INSERT },
+  { "left",  BX_KEY_LEFT },
+  { "menu",  BX_KEY_MENU },
+  { "minus", BX_KEY_MINUS },
+  { "pgdwn", BX_KEY_PAGE_DOWN },
+  { "pgup",  BX_KEY_PAGE_UP },
+  { "plus",  BX_KEY_KP_ADD },
+  { "right", BX_KEY_RIGHT },
+  { "shift", BX_KEY_SHIFT_L },
+  { "space", BX_KEY_SPACE },
+  { "tab",   BX_KEY_TAB },
+  { "up",    BX_KEY_UP },
+  { "win",   BX_KEY_WIN_L }
+};
+
 bx_gui_c::bx_gui_c(void)
 {
   put("GUI"); // Init in specific_init
   settype(GUILOG);
+  statusitem_count = 0;
+  framebuffer = NULL;
 }
 
 bx_gui_c::~bx_gui_c()
 {
+  if (framebuffer != NULL) {
+    delete [] framebuffer;
+  }
 }
 
   void
 bx_gui_c::init(int argc, char **argv, unsigned tilewidth, unsigned tileheight)
 {
+  BX_GUI_THIS new_gfx_api = 0;
+  BX_GUI_THIS host_xres = 640;
+  BX_GUI_THIS host_yres = 480;
+  BX_GUI_THIS host_bpp = 8;
+  BX_GUI_THIS dialog_caps = BX_GUI_DLG_RUNTIME;
+
   specific_init(argc, argv, tilewidth, tileheight, BX_HEADER_BAR_Y);
 
   // Define some bitmaps to use in the headerbar
@@ -167,6 +225,9 @@ bx_gui_c::init(int argc, char **argv, unsigned tilewidth, unsigned tileheight)
 
   BX_GUI_THIS charmap_updated = 0;
 
+  if (!BX_GUI_THIS new_gfx_api) {
+    BX_GUI_THIS framebuffer = new Bit8u[BX_MAX_XRES * BX_MAX_YRES * 4];
+  }
   show_headerbar();
 }
 
@@ -214,9 +275,7 @@ bx_gui_c::floppyA_handler(void)
 {
   if (bx_options.floppya.Odevtype->get() == BX_FLOPPY_NONE)
     return; // no primary floppy device present
-#ifdef WIN32
-  if (strcmp(bx_options.Osel_displaylib->get_choice(bx_options.Osel_displaylib->get()),
-              "rfb")) {
+  if (BX_GUI_THIS dialog_caps & BX_GUI_DLG_FLOPPY) {
     // instead of just toggling the status, call win32dialog to bring up
     // a dialog asking what disk image you want to switch to.
     int ret = SIM->ask_param (BXP_FLOPPYA_PATH);
@@ -225,7 +284,6 @@ bx_gui_c::floppyA_handler(void)
     }
     return;
   }
-#endif
   BX_GUI_THIS floppyA_status = !BX_GUI_THIS floppyA_status;
   DEV_floppy_set_media_status(0, BX_GUI_THIS floppyA_status);
   BX_GUI_THIS update_drive_status_buttons ();
@@ -236,9 +294,7 @@ bx_gui_c::floppyB_handler(void)
 {
   if (bx_options.floppyb.Odevtype->get() == BX_FLOPPY_NONE)
     return; // no secondary floppy device present
-#ifdef WIN32
-  if (strcmp(bx_options.Osel_displaylib->get_choice(bx_options.Osel_displaylib->get()),
-              "rfb")) {
+  if (BX_GUI_THIS dialog_caps & BX_GUI_DLG_FLOPPY) {
     // instead of just toggling the status, call win32dialog to bring up
     // a dialog asking what disk image you want to switch to.
     int ret = SIM->ask_param (BXP_FLOPPYB_PATH);
@@ -247,7 +303,6 @@ bx_gui_c::floppyB_handler(void)
     }
     return;
   }
-#endif
   BX_GUI_THIS floppyB_status = !BX_GUI_THIS floppyB_status;
   DEV_floppy_set_media_status(1, BX_GUI_THIS floppyB_status);
   BX_GUI_THIS update_drive_status_buttons ();
@@ -257,30 +312,22 @@ bx_gui_c::floppyB_handler(void)
 bx_gui_c::cdromD_handler(void)
 {
   Bit32u handle = DEV_hd_get_first_cd_handle();
-  if (!strcmp(bx_options.Osel_config->get_choice(bx_options.Osel_config->get()),
-              "wx")) {
-    // instead of just toggling the status, call wxWindows to bring up 
+  if (BX_GUI_THIS dialog_caps & BX_GUI_DLG_CDROM) {
+    // instead of just toggling the status, call win32dialog to bring up 
     // a dialog asking what disk image you want to switch to.
-    // BBD: for now, find the first cdrom and call ask_param on that.
-    // Since we could have multiple cdroms now, maybe we should be adding
-    // one cdrom button for each?
+    // This code handles the first cdrom only. The cdrom drives #2, #3 and
+    // #4 are handled in the win32 runtime dialog.
     bx_param_c *cdrom = SIM->get_first_cdrom ();
     if (cdrom == NULL)
       return;  // no cdrom found
     int ret = SIM->ask_param (cdrom->get_id ());
-    if (ret < 0) return;  // cancelled
-    // eject and then insert the disk.  If the new path is invalid,
-    // the status will return 0.
-    unsigned status = DEV_hd_set_cd_media_status(handle, 0);
-    printf ("eject disk, new_status is %d\n", status);
-    status = DEV_hd_set_cd_media_status(handle, 1);
-    printf ("insert disk, new_status is %d\n", status);
-    fflush (stdout);
-    BX_GUI_THIS cdromD_status = status;
-  } else {
-    BX_GUI_THIS cdromD_status =
-      DEV_hd_set_cd_media_status(handle, !BX_GUI_THIS cdromD_status);
+    if (ret > 0) {
+      BX_GUI_THIS update_drive_status_buttons ();
+    }
+    return;
   }
+  BX_GUI_THIS cdromD_status =
+    DEV_hd_set_cd_media_status(handle, !BX_GUI_THIS cdromD_status);
   BX_GUI_THIS update_drive_status_buttons ();
 }
 
@@ -288,9 +335,7 @@ bx_gui_c::cdromD_handler(void)
 bx_gui_c::reset_handler(void)
 {
   BX_INFO(( "system RESET callback." ));
-  bx_pc_system.ResetSignal( PCS_SET ); /* XXX is this right? */
-  for (int i=0; i<BX_SMP_PROCESSORS; i++)
-      BX_CPU(i)->reset(BX_RESET_HARDWARE);
+  bx_pc_system.Reset( BX_RESET_HARDWARE );
 }
 
   void
@@ -416,13 +461,7 @@ bx_gui_c::snapshot_handler(void)
   }
   //FIXME
   char filename[BX_PATHNAME_LEN];
-#ifdef WIN32
-  if (strcmp(bx_options.Osel_displaylib->get_choice(bx_options.Osel_displaylib->get()),
-              "rfb")) {
-#else
-  if (!strcmp(bx_options.Osel_config->get_choice(bx_options.Osel_config->get()),
-              "wx")) {
-#endif
+  if (BX_GUI_THIS dialog_caps & BX_GUI_DLG_SNAPSHOT) {
     int ret = SIM->ask_filename (filename, sizeof(filename),
                                  "Save snapshot as...", "snapshot.txt",
                                  bx_param_string_c::SAVE_FILE_DIALOG);
@@ -462,8 +501,7 @@ bx_gui_c::paste_handler(void)
   void
 bx_gui_c::config_handler(void)
 {
-  if (strcmp(bx_options.Osel_displaylib->get_choice(bx_options.Osel_displaylib->get()),
-              "rfb")) {
+  if (BX_GUI_THIS dialog_caps & BX_GUI_DLG_RUNTIME) {
     SIM->configuration_interface (NULL, CI_RUNTIME_CONFIG);
   }
 }
@@ -476,59 +514,77 @@ bx_gui_c::toggle_mouse_enable(void)
   bx_options.Omouse_enabled->set (!old);
 }
 
+Bit32u get_user_key(char *key)
+{
+  int i = 0;
+
+  while (i < N_USER_KEYS) {
+    if (!strcmp(key, user_keys[i].key))
+      return user_keys[i].symbol;
+    i++;
+  }
+  return BX_KEY_UNKNOWN;
+}
+
   void
 bx_gui_c::userbutton_handler(void)
 {
-  unsigned shortcut[4];
-  unsigned p;
-  char *user_shortcut;
-  int i, len, ret = 1;
+  Bit32u shortcut[4];
+  Bit32u symbol;
+  char user_shortcut[512];
+  char *ptr;
+  int i, len = 0, ret = 1;
 
-  len = 0;
-#ifdef WIN32
-  if (strcmp(bx_options.Osel_displaylib->get_choice(bx_options.Osel_displaylib->get()),
-              "rfb")) {
-#else
-  if (!strcmp(bx_options.Osel_config->get_choice(bx_options.Osel_config->get()),
-              "wx")) {
-#endif
+  if (BX_GUI_THIS dialog_caps & BX_GUI_DLG_USER) {
     ret = SIM->ask_param (BXP_USER_SHORTCUT);
   }
-  user_shortcut = bx_options.Ouser_shortcut->getptr();
+  strcpy(user_shortcut, bx_options.Ouser_shortcut->getptr());
   if ((ret > 0) && user_shortcut[0] && (strcmp(user_shortcut, "none"))) {
-    len = 0;
-    p = 0;
-    while ((p < strlen(user_shortcut)) && (len < 3)) {
-      if (!strncmp(user_shortcut+p, "alt", 3)) {
-        shortcut[len++] = BX_KEY_ALT_L;
-        p += 3;
-      } else if (!strncmp(user_shortcut+p, "ctrl", 4)) {
-        shortcut[len++] = BX_KEY_CTRL_L;
-        p += 4;
-      } else if (!strncmp(user_shortcut+p, "del", 3)) {
-        shortcut[len++] = BX_KEY_DELETE;
-        p += 3;
-      } else if (!strncmp(user_shortcut+p, "esc", 3)) {
-        shortcut[len++] = BX_KEY_ESC;
-        p += 3;
-      } else if (!strncmp(user_shortcut+p, "f1", 2)) {
-        shortcut[len++] = BX_KEY_F1;
-        p += 2;
-      } else if (!strncmp(user_shortcut+p, "f4", 2)) {
-        shortcut[len++] = BX_KEY_F4;
-        p += 2;
-      } else if (!strncmp(user_shortcut+p, "tab", 3)) {
-        shortcut[len++] = BX_KEY_TAB;
-        p += 3;
-      } else if (!strncmp(user_shortcut+p, "win", 3)) {
-        shortcut[len++] = BX_KEY_WIN_L;
-        p += 3;
-      } else if (!strncmp(user_shortcut+p, "bksp", 4)) {
-        shortcut[len++] = BX_KEY_BACKSPACE;
-        p += 4;
-      } else {
-        BX_ERROR(("Unknown shortcut %s ignored", user_shortcut));
-        return;
+    ptr = strtok(user_shortcut, "-");
+    if ((strcmp(ptr, bx_options.Ouser_shortcut->getptr())) ||
+        (strlen(bx_options.Ouser_shortcut->getptr()) < 6)) {
+      while (ptr) {
+        symbol = get_user_key(ptr);
+        if (symbol == BX_KEY_UNKNOWN) {
+          BX_ERROR(("Unknown shortcut %s ignored", ptr));
+          return;
+        }
+        shortcut[len++] = symbol;
+        ptr = strtok(NULL, "-");
+      }
+    } else {
+      while ((ptr[0]) && (len < 3)) {
+        if (!strncmp(ptr, "alt", 3)) {
+          shortcut[len++] = BX_KEY_ALT_L;
+          ptr += 3;
+        } else if (!strncmp(ptr, "ctrl", 4)) {
+          shortcut[len++] = BX_KEY_CTRL_L;
+          ptr += 4;
+        } else if (!strncmp(ptr, "del", 3)) {
+          shortcut[len++] = BX_KEY_DELETE;
+          ptr += 3;
+        } else if (!strncmp(ptr, "esc", 3)) {
+          shortcut[len++] = BX_KEY_ESC;
+          ptr += 3;
+        } else if (!strncmp(ptr, "f1", 2)) {
+          shortcut[len++] = BX_KEY_F1;
+          ptr += 2;
+        } else if (!strncmp(ptr, "f4", 2)) {
+          shortcut[len++] = BX_KEY_F4;
+          ptr += 2;
+        } else if (!strncmp(ptr, "tab", 3)) {
+          shortcut[len++] = BX_KEY_TAB;
+          ptr += 3;
+        } else if (!strncmp(ptr, "win", 3)) {
+          shortcut[len++] = BX_KEY_WIN_L;
+          ptr += 3;
+        } else if (!strncmp(ptr, "bksp", 4)) {
+          shortcut[len++] = BX_KEY_BACKSPACE;
+          ptr += 4;
+        } else {
+          BX_ERROR(("Unknown shortcut %s ignored", user_shortcut));
+          return;
+        }
       }
     }
     i = 0;
@@ -590,3 +646,152 @@ bx_gui_c::set_text_charbyte(Bit16u address, Bit8u data)
   BX_GUI_THIS char_changed[address >> 5] = 1;
   BX_GUI_THIS charmap_updated = 1;
 }
+
+  
+  void
+bx_gui_c::beep_on(float frequency)
+{
+  BX_INFO(( "GUI Beep ON (frequency=%.2f)",frequency));
+}
+
+  void
+bx_gui_c::beep_off()
+{
+  BX_INFO(( "GUI Beep OFF"));
+}
+
+  int
+bx_gui_c::register_statusitem(const char *text)
+{
+  if (statusitem_count < BX_MAX_STATUSITEMS) {
+    strncpy(statusitem_text[statusitem_count], text, 8);
+    statusitem_text[statusitem_count][7] = 0;
+    return statusitem_count++;
+  } else {
+   return -1;
+  }
+}
+
+  void
+bx_gui_c::get_capabilities(Bit16u *xres, Bit16u *yres, Bit16u *bpp)
+{
+  *xres = 1024;
+  *yres = 768;
+  *bpp = 32;
+}
+
+  bx_svga_tileinfo_t *
+bx_gui_c::graphics_tile_info(bx_svga_tileinfo_t *info)
+{
+  if (!info) {
+    info = (bx_svga_tileinfo_t *)malloc(sizeof(bx_svga_tileinfo_t));
+    if (!info) {
+      return NULL;
+    }
+  }
+
+  BX_GUI_THIS host_pitch = BX_GUI_THIS host_xres * ((BX_GUI_THIS host_bpp + 1) >> 3);
+
+  info->bpp = BX_GUI_THIS host_bpp;
+  info->pitch = BX_GUI_THIS host_pitch;
+  switch (info->bpp) {
+    case 15:
+      info->red_shift = 15;
+      info->green_shift = 10;
+      info->blue_shift = 5;
+      info->red_mask = 0x7c00;
+      info->green_mask = 0x03e0;
+      info->blue_mask = 0x001f;
+      break;
+    case 16:
+      info->red_shift = 16;
+      info->green_shift = 11;
+      info->blue_shift = 5;
+      info->red_mask = 0xf800;
+      info->green_mask = 0x07e0;
+      info->blue_mask = 0x001f;
+      break;
+    case 24:
+    case 32:
+      info->red_shift = 24;
+      info->green_shift = 16;
+      info->blue_shift = 8;
+      info->red_mask = 0xff0000;
+      info->green_mask = 0x00ff00;
+      info->blue_mask = 0x0000ff;
+      break;
+  }
+  info->is_indexed = (BX_GUI_THIS host_bpp == 8);
+#ifdef BX_LITTLE_ENDIAN
+  info->is_little_endian = 1;
+#else
+  info->is_little_endian = 0;
+#endif
+
+  return info;
+}
+
+  Bit8u *
+bx_gui_c::graphics_tile_get(unsigned x0, unsigned y0,
+                            unsigned *w, unsigned *h)
+{
+  if (x0+X_TILESIZE > BX_GUI_THIS host_xres) {
+    *w = BX_GUI_THIS host_xres - x0;
+  }
+  else {
+    *w = X_TILESIZE;
+  }
+
+  if (y0+Y_TILESIZE > BX_GUI_THIS host_yres) {
+    *h = BX_GUI_THIS host_yres - y0;
+  }
+  else {
+    *h = Y_TILESIZE;
+  }
+
+  return (Bit8u *)framebuffer + y0 * BX_GUI_THIS host_pitch +
+                  x0 * ((BX_GUI_THIS host_bpp + 1) >> 3);
+}
+
+  void
+bx_gui_c::graphics_tile_update_in_place(unsigned x0, unsigned y0,
+                                        unsigned w, unsigned h)
+{
+  Bit8u tile[X_TILESIZE * Y_TILESIZE * 4];
+  Bit8u *tile_ptr, *fb_ptr;
+  Bit16u xc, yc, fb_pitch, tile_pitch;
+  Bit8u r, diffx, diffy;
+
+  diffx = (x0 % X_TILESIZE);
+  diffy = (y0 % Y_TILESIZE);
+  if (diffx > 0) {
+    x0 -= diffx;
+    w += diffx;
+  }
+  if (diffy > 0) {
+    y0 -= diffy;
+    h += diffy;
+  }
+  fb_pitch = BX_GUI_THIS host_pitch;
+  tile_pitch = X_TILESIZE * ((BX_GUI_THIS host_bpp + 1) >> 3);
+  for (yc=y0; yc<(y0+h); yc+=Y_TILESIZE) {
+    for (xc=x0; xc<(x0+w); xc+=X_TILESIZE) {
+      fb_ptr = BX_GUI_THIS framebuffer + (yc * fb_pitch + xc * ((BX_GUI_THIS host_bpp + 1) >> 3));
+      tile_ptr = &tile[0];
+      for (r=0; r<h; r++) {
+        memcpy(tile_ptr, fb_ptr, tile_pitch);
+        fb_ptr += fb_pitch;
+        tile_ptr += tile_pitch;
+      }
+      BX_GUI_THIS graphics_tile_update(tile, xc, yc);
+    }
+  }
+}
+
+  void 
+bx_gui_c::show_ips(Bit32u ips_count)
+{
+#if BX_SHOW_IPS
+  BX_INFO(("ips = %u", ips_count));
+#endif
+}
