@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////
-// $Id: wx.cc,v 1.65 2003/08/23 17:53:27 vruppert Exp $
+// $Id: wx.cc,v 1.76 2005/11/12 16:09:55 vruppert Exp $
 /////////////////////////////////////////////////////////////////
 //
-// wxWindows VGA display for Bochs.  wx.cc implements a custom
+// wxWidgets VGA display for Bochs.  wx.cc implements a custom
 // wxPanel called a MyPanel, which has methods to display
 // text and VGA graphics on the panel.  Normally, a MyPanel
 // is instantiated within a MyFrame created by wxmain.cc, but
@@ -10,14 +10,14 @@
 //
 // The separation between wxmain.cc and wx.cc is as follows:
 // - wxmain.cc implements a Bochs configuration interface (CI),
-//   which is the wxWindows equivalent of control.cc.  wxmain creates
+//   which is the wxWidgets equivalent of textconfig.cc. wxmain creates
 //   a frame with several menus and a toolbar, and allows the user to
 //   choose the machine configuration and start the simulation.  Note
 //   that wxmain.cc does NOT include bochs.h.  All interactions
 //   between the CI and the simulator are through the siminterface
 //   object.
-// - wx.cc implements a VGA display screen using wxWindows.  It is 
-//   is the wxWindows equivalent of x.cc, win32.cc, macos.cc, etc.
+// - wx.cc implements a VGA display screen using wxWidgets.  It is 
+//   is the wxWidgets equivalent of x.cc, win32.cc, macos.cc, etc.
 //   wx.cc includes bochs.h and has access to all Bochs devices.
 //   The VGA panel accepts only paint, key, and mouse events.  As it
 //   receives events, it builds BxEvents and places them into a 
@@ -37,6 +37,7 @@
 #define BX_PLUGGABLE
 
 #include "bochs.h"
+#include "iodev.h"
 #if BX_WITH_WX
 
 // For compilers that support precompilation, includes <wx/wx.h>.
@@ -65,6 +66,8 @@ class bx_wx_gui_c : public bx_gui_c {
 public:
   bx_wx_gui_c (void) {}
   DECLARE_GUI_VIRTUAL_METHODS()
+  DECLARE_GUI_NEW_VIRTUAL_METHODS()
+  virtual void statusbar_setitem(int element, bx_bool active);
 };
 
 // declare one instance of the gui object and call macro to insert the
@@ -88,14 +91,15 @@ static char *wxScreen = NULL;
 wxCriticalSection wxScreen_lock;
 static long wxScreenX = 0;
 static long wxScreenY = 0;
-static long wxTileX = 0;
-static long wxTileY = 0;
+static unsigned wxTileX = 0;
+static unsigned wxTileY = 0;
 static unsigned long wxCursorX = 0;
 static unsigned long wxCursorY = 0;
 static unsigned long wxFontX = 0;
 static unsigned long wxFontY = 0;
 static unsigned int text_rows=25, text_cols=80;
 static Bit8u h_panning = 0, v_panning = 0;
+static Bit16u line_compare = 1023;
 static unsigned vga_bpp=8;
 struct {
   unsigned char red;
@@ -105,7 +109,9 @@ struct {
 wxCriticalSection event_thread_lock;
 BxEvent event_queue[MAX_EVENTS];
 unsigned long num_events = 0;
+#if defined (wxHAS_RAW_KEY_CODES) && defined(__WXGTK__)
 static Bit32u convertStringToGDKKey (const char *string);
+#endif
 
 
 //////////////////////////////////////////////////////////////
@@ -142,7 +148,7 @@ MyPanel::~MyPanel ()
   thePanel = NULL;
 }
 
-void MyPanel::OnTimer(wxCommandEvent& WXUNUSED(event))
+void MyPanel::OnTimer(wxTimerEvent& WXUNUSED(event))
 {
   IFDBG_VGA(wxLogDebug (wxT ("timer")));
   if (needRefresh) {
@@ -164,7 +170,7 @@ void MyPanel::OnPaint(wxPaintEvent& WXUNUSED(event))
     wxPoint pt = GetClientAreaOrigin();
     wxImage screenImage(wxScreenX, wxScreenY, (unsigned char *)wxScreen, TRUE);
     IFDBG_VGA(wxLogDebug (wxT ("drawBitmap")));
-    dc.DrawBitmap(screenImage.ConvertToBitmap(), pt.x, pt.y, FALSE);
+    dc.DrawBitmap(wxBitmap(screenImage), pt.x, pt.y, FALSE);
   }
   needRefresh = false;
 }
@@ -183,7 +189,7 @@ void MyPanel::ToggleMouse (bool fromToolbar)
     wxString msg = 
       "You have enabled the mouse in Bochs, so now your mouse actions will\n"
       "be sent into the simulator.  The usual mouse cursor will be trapped\n"
-      "inside the Bochs window until you press F12 or press the middle button\n"
+      "inside the Bochs window until you press a CTRL key + the middle button\n"
       "to turn mouse capture off.";
     wxMessageBox(msg, "Mouse Capture Enabled", wxOK | wxICON_INFORMATION);
     first_enable = false;
@@ -219,7 +225,7 @@ void MyPanel::OnMouse(wxMouseEvent& event)
     }
   )
 
-  if (event.MiddleDown ()) {
+  if (event.MiddleDown () && event.ControlDown ()) {
     ToggleMouse (false);
     return;
   }
@@ -231,6 +237,7 @@ void MyPanel::OnMouse(wxMouseEvent& event)
   Bit32u buttons;
   buttons  = event.LeftIsDown ()  ? 1 : 0;
   buttons |= event.RightIsDown () ? 2 : 0;
+  buttons |= event.MiddleIsDown () ? 4 : 0;
   if (x==mouseSavedX && y==mouseSavedY && !event.IsButton ()) {
     // nothing happened.  This could have been generated by the WarpPointer.
     return;
@@ -270,10 +277,6 @@ MyPanel::MyRefresh ()
 
 void MyPanel::OnKeyDown(wxKeyEvent& event)
 {
-  if(event.GetKeyCode() == WXK_F12) {
-    ToggleMouse (false);
-    return;
-  }
   wxCriticalSectionLocker lock(event_thread_lock);
   if(num_events < MAX_EVENTS) {
     event_queue[num_events].type = BX_ASYNC_EVT_KEY;
@@ -736,10 +739,10 @@ bx_bool
 MyPanel::fillBxKeyEvent (wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool release)
 {
   // Use raw codes if they are available.  Raw codes are a nonstandard addition
-  // to the wxWindows library.  At present, the only way to use the "RAW_CODES"
-  // mode is to apply Bryce's "patch.wx-raw-keycodes" patch to the wxWindows
+  // to the wxWidgets library.  At present, the only way to use the "RAW_CODES"
+  // mode is to apply Bryce's "patch.wx-raw-keycodes" patch to the wxWidgets
   // sources and recompile.  This patch, or something like it, should appear in
-  // future wxWindows versions.
+  // future wxWidgets versions.
 
 #if defined (wxHAS_RAW_KEY_CODES) && defined(__WXMSW__)
   return fillBxKeyEvent_MSW (wxev, bxev, release);
@@ -909,6 +912,9 @@ bx_wx_gui_c::specific_init(int argc, char **argv, unsigned tilewidth, unsigned t
 #else
     bx_keymap.loadKeymap(NULL);
 #endif
+
+  new_gfx_api = 1;
+  dialog_caps = BX_GUI_DLG_USER | BX_GUI_DLG_SNAPSHOT;
 }
 
 // ::HANDLE_EVENTS()
@@ -1026,6 +1032,42 @@ void bx_wx_gui_c::handle_events(void)
   num_events = 0;
 }
 
+void 
+bx_wx_gui_c::statusbar_setitem(int element, bx_bool active)
+{
+#if defined( __WXMSW__)
+  char status_text[10];
+#endif
+
+  if (element < 0) {
+    for (unsigned i = 0; i < statusitem_count; i++) {
+      if (active) {
+#if defined( __WXMSW__)
+        status_text[0] = 9;
+        strcpy(status_text+1, statusitem_text[i]);
+        theFrame->SetStatusText(status_text, i+1);
+#else
+        theFrame->SetStatusText(statusitem_text[i], i+1);
+#endif
+      } else {
+        theFrame->SetStatusText("", i+1);
+      }
+    }
+  } else if ((unsigned)element < statusitem_count) {
+    if (active) {
+#if defined( __WXMSW__)
+        status_text[0] = 9;
+        strcpy(status_text+1, statusitem_text[element]);
+        theFrame->SetStatusText(status_text, element+1);
+#else
+      theFrame->SetStatusText(statusitem_text[element], element+1);
+#endif
+    } else {
+      theFrame->SetStatusText("", element+1);
+    }
+  }
+}
+
 // ::FLUSH()
 //
 // Called periodically, requesting that the gui code flush all pending
@@ -1055,70 +1097,12 @@ bx_wx_gui_c::clear_screen(void)
 static void 
 UpdateScreen(unsigned char *newBits, int x, int y, int width, int height) 
 {
-  Bit16u *newBits16 = (Bit16u *)newBits;
-
   IFDBG_VGA(wxLogDebug (wxT ("MyPanel::UpdateScreen trying to get lock. wxScreen=%p", wxScreen)));
   wxCriticalSectionLocker lock(wxScreen_lock);
   IFDBG_VGA(wxLogDebug (wxT ("MyPanel::UpdateScreen got lock. wxScreen=%p", wxScreen)));
   if(wxScreen != NULL) {
     switch (vga_bpp) {
-      case 32:
-        for(int i = 0; i < height; i++) {
-          char *pwxScreen = &wxScreen[(y * wxScreenX * 3) + (x * 3)];
-          for(int c = 0; c < width; c++) {
-            unsigned pixel = ((i * width) + c) * 4;
-            pwxScreen[0] = newBits[pixel+2];
-            pwxScreen[1] = newBits[pixel+1];
-            pwxScreen[2] = newBits[pixel];
-            pwxScreen += 3;
-          }
-          y++;
-          if(y >= wxScreenY) break;
-        }
-        break;
-      case 24:
-        for(int i = 0; i < height; i++) {
-          char *pwxScreen = &wxScreen[(y * wxScreenX * 3) + (x * 3)];
-          for(int c = 0; c < width; c++) {
-            unsigned pixel = ((i * width) + c) * 3;
-            pwxScreen[0] = newBits[pixel+2];
-            pwxScreen[1] = newBits[pixel+1];
-            pwxScreen[2] = newBits[pixel];
-            pwxScreen += 3;
-          }
-          y++;
-          if(y >= wxScreenY) break;
-        }
-        break;
-      case 16:
-        for(int i = 0; i < height; i++) {
-          char *pwxScreen = &wxScreen[(y * wxScreenX * 3) + (x * 3)];
-          for(int c = 0; c < width; c++) {
-            unsigned pixel = (i * width) + c;
-            pwxScreen[0] = (newBits16[pixel] & 0xF800) >> 8;
-            pwxScreen[1] = (newBits16[pixel] & 0x07e0) >> 3;
-            pwxScreen[2] = (newBits16[pixel] & 0x001f) << 3;
-            pwxScreen += 3;
-          }
-          y++;
-          if(y >= wxScreenY) break;
-        }
-        break;
-      case 15:
-        for(int i = 0; i < height; i++) {
-          char *pwxScreen = &wxScreen[(y * wxScreenX * 3) + (x * 3)];
-          for(int c = 0; c < width; c++) {
-            unsigned pixel = (i * width) + c;
-            pwxScreen[0] = (newBits16[pixel] & 0x7c00) >> 7;
-            pwxScreen[1] = (newBits16[pixel] & 0x03e0) >> 2;
-            pwxScreen[2] = (newBits16[pixel] & 0x001f) << 3;
-            pwxScreen += 3;
-          }
-          y++;
-          if(y >= wxScreenY) break;
-        }
-        break;
-      default: /* 8 bpp */
+      case 8: /* 8 bpp */
         for(int i = 0; i < height; i++) {
           char *pwxScreen = &wxScreen[(y * wxScreenX * 3) + (x * 3)];
           for(int c = 0; c < width; c++) {
@@ -1131,6 +1115,10 @@ UpdateScreen(unsigned char *newBits, int x, int y, int width, int height)
           y++;
           if(y >= wxScreenY) break;
         }
+        break;
+      default:
+        BX_PANIC(("%u bpp modes handled by new graphics API", vga_bpp));
+        return;
     }
   } else {
     IFDBG_VGA (wxLogDebug (wxT ("UpdateScreen with null wxScreen")));
@@ -1146,6 +1134,8 @@ DrawBochsBitmap(int x, int y, int width, int height, char *bmap, char color, int
   char bgcolor = DEV_vga_get_actl_pal_idx((color >> 4) & 0xF);
   char fgcolor = DEV_vga_get_actl_pal_idx(color & 0xF);
 
+  if (y > wxScreenY) return;
+
   for(int i = 0; i < bytes; i+=width) {
     mask = 0x80 >> fontx;
     for(int j = 0; j < width; j++) {
@@ -1192,11 +1182,12 @@ void bx_wx_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
 {
   IFDBG_VGA(wxLogDebug (wxT ("text_update")));
 
-  unsigned char *old_line, *new_line;
-  unsigned char cAttr, cChar;
-  unsigned int curs, hchars, offset, rows, x, y, xc, yc, yc2;
+  Bit8u *old_line, *new_line, *text_base;
+  Bit8u cAttr, cChar;
+  unsigned int curs, hchars, offset, rows, x, y, xc, yc, yc2, cs_y;
   Bit8u cfwidth, cfheight, cfheight2, font_col, font_row, font_row2;
-  bx_bool forceUpdate = 0, gfxchar;
+  Bit8u split_textrow, split_fontrows;
+  bx_bool forceUpdate = 0, gfxchar, split_screen;
 
   UNUSED(nrows);
   if(charmap_updated) {
@@ -1209,6 +1200,10 @@ void bx_wx_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
     h_panning = tm_info.h_panning;
     v_panning = tm_info.v_panning;
   }
+  if(tm_info.line_compare != line_compare) {
+    forceUpdate = 1;
+    line_compare = tm_info.line_compare;
+  }
 
   // first invalidate character at previous and new cursor location
   if((wxCursorY < text_rows) && (wxCursorX < text_cols)) {
@@ -1226,10 +1221,24 @@ void bx_wx_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
   rows = text_rows;
   if (v_panning) rows++;
   y = 0;
+  cs_y = 0;
+  text_base = new_text - tm_info.start_address;
+  split_textrow = (line_compare + v_panning) / wxFontY;
+  split_fontrows = ((line_compare + v_panning) % wxFontY) + 1;
+  split_screen = 0;
   do {
     hchars = text_cols;
     if (h_panning) hchars++;
-    if (v_panning) {
+    if (split_screen) {
+      yc = line_compare + cs_y * wxFontY + 1;
+      font_row = 0;
+      if (rows == 1) {
+        cfheight = (wxScreenY - line_compare - 1) % wxFontY;
+        if (cfheight == 0) cfheight = wxFontY;
+      } else {
+        cfheight = wxFontY;
+      }
+    } else if (v_panning) {
       if (y == 0) {
         yc = 0;
         font_row = v_panning;
@@ -1248,10 +1257,13 @@ void bx_wx_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
       font_row = 0;
       cfheight = wxFontY;
     }
+    if (!split_screen && (y == split_textrow)) {
+      if (split_fontrows < cfheight) cfheight = split_fontrows;
+    }
     new_line = new_text;
     old_line = old_text;
     x = 0;
-    offset = y * tm_info.line_offset;
+    offset = cs_y * tm_info.line_offset;
     do {
       if (h_panning) {
         if (hchars > text_cols) {
@@ -1305,11 +1317,22 @@ void bx_wx_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
       old_text+=2;
       offset+=2;
     } while (--hchars);
-    y++;
-    new_text = new_line + tm_info.line_offset;
-    old_text = old_line + tm_info.line_offset;
+    if (!split_screen && (y == split_textrow)) {
+      new_text = text_base;
+      forceUpdate = 1;
+      cs_y = 0;
+      if (tm_info.split_hpanning) h_panning = 0;
+      rows = ((wxScreenY - line_compare + wxFontY - 2) / wxFontY) + 1;
+      split_screen = 1;
+    } else {
+      y++;
+      cs_y++;
+      new_text = new_line + tm_info.line_offset;
+      old_text = old_line + tm_info.line_offset;
+    }
   } while (--rows);
 
+  h_panning = tm_info.h_panning;
   wxCursorX = cursor_x;
   wxCursorY = cursor_y;
 
@@ -1353,12 +1376,62 @@ bx_wx_gui_c::palette_change(unsigned index, unsigned red, unsigned green, unsign
 void bx_wx_gui_c::graphics_tile_update(Bit8u *tile, unsigned x0, unsigned y0)
 {
   IFDBG_VGA (wxLogDebug (wxT ("graphics_tile_update")));
-  //static Bit32u counter = 0;
-  //BX_INFO (("graphics_tile_update executed %d times", ++counter));
   UpdateScreen(tile, x0, y0, wxTileX, wxTileY);
   thePanel->MyRefresh ();
 }
 
+  bx_svga_tileinfo_t *
+bx_wx_gui_c::graphics_tile_info(bx_svga_tileinfo_t *info)
+{
+  if (!info) {
+    info = (bx_svga_tileinfo_t *)malloc(sizeof(bx_svga_tileinfo_t));
+    if (!info) {
+      return NULL;
+    }
+  }
+
+  info->bpp = 24;
+  info->pitch = wxScreenX * 3;
+  info->red_shift = 8;
+  info->green_shift = 16;
+  info->blue_shift = 24;
+  info->red_mask = 0x0000ff;
+  info->green_mask = 0x00ff00;
+  info->blue_mask = 0xff0000;
+  info->is_indexed = 0;
+  info->is_little_endian = 1;
+
+  return info;
+};
+
+  Bit8u *
+bx_wx_gui_c::graphics_tile_get(unsigned x0, unsigned y0,
+                            unsigned *w, unsigned *h)
+{
+  if (x0+wxTileX > (unsigned)wxScreenX) {
+    *w = wxScreenX - x0;
+  }
+  else {
+    *w = wxTileX;
+  }
+
+  if (y0+wxTileY > (unsigned)wxScreenY) {
+    *h = wxScreenY - y0;
+  }
+  else {
+    *h = wxTileY;
+  }
+
+  return (Bit8u *)wxScreen + y0 * wxScreenX * 3 + x0 * 3;
+}
+
+  void
+bx_wx_gui_c::graphics_tile_update_in_place(unsigned x0, unsigned y0,
+                                        unsigned w, unsigned h)
+{
+  thePanel->MyRefresh ();
+}
+
 // ::DIMENSION_UPDATE()
 //
 // Called from the simulator when the VGA mode changes it's X,Y dimensions.
@@ -1378,7 +1451,7 @@ void bx_wx_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight, uns
   IFDBG_VGA(wxLogDebug (wxT ("MyPanel::dimension_update got lock. wxScreen=%p", wxScreen)));
   BX_INFO (("dimension update x=%d y=%d fontheight=%d fontwidth=%d bpp=%d", x, y, fheight, fwidth, bpp));
   if ((bpp == 8) || (bpp == 15) || (bpp == 16) || (bpp == 24) || (bpp == 32)) {
-    if (bpp == 32) BX_INFO(("wxWindows ignores bit 24..31 in 32bpp mode"));
+    if (bpp == 32) BX_INFO(("wxWidgets ignores bit 24..31 in 32bpp mode"));
     vga_bpp = bpp;
   }
   else
@@ -1399,7 +1472,7 @@ void bx_wx_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight, uns
   IFDBG_VGA(wxLogDebug (wxT ("MyPanel::dimension_update gave up lock. wxScreen=%p", wxScreen)));
   // Note: give up wxScreen_lock before calling SetClientSize.  I did
   // this because I was sometimes seeing thread deadlock in win32, apparantly 
-  // related to wxStreen_lock.  The wxWindows GUI thread was sitting in OnPaint
+  // related to wxStreen_lock.  The wxWidgets GUI thread was sitting in OnPaint
   // trying to get the wxScreen_lock, and the simulation thread was stuck in some
   // native win32 function called by SetClientSize (below).  As with many
   // thread problems, it happened sporadically so it's hard to prove that this
@@ -1530,7 +1603,7 @@ bx_wx_gui_c::get_clipboard_text(Bit8u **bytes, Bit32s *nbytes)
       // buf will be freed in bx_keyb_c::paste_bytes or 
       // bx_keyb_c::service_paste_buf, using delete [].
     } else {
-      BX_ERROR (("paste: could not open wxWindows clipboard"));
+      BX_ERROR (("paste: could not open wxWidgets clipboard"));
     }
     wxTheClipboard->Close ();
   }
