@@ -1,32 +1,37 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: ioapic.h,v 1.8.2.1 2005/07/06 20:58:53 vruppert Exp $
+// $Id: ioapic.h,v 1.11 2005/12/26 19:42:09 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 
 extern class bx_ioapic_c bx_ioapic;
 
-#define BX_IOAPIC_VERSION_ID 0x00170011  // same version as 82093 IOAPIC
-#define BX_IOAPIC_NUM_PINS 0x18
+#define BX_IOAPIC_NUM_PINS   (0x18)
+
+// use the same version as 82093 IOAPIC (0x00170011)
+#define BX_IOAPIC_VERSION_ID (((BX_IOAPIC_NUM_PINS - 1) << 16) | 0x11)
 
 class bx_io_redirect_entry_t {
-  Bit64u value;
+  Bit32u hi, lo;
+
 public:
-  Bit32u get_even_word () { return (Bit32u)(value & 0xffffffff); }
-  Bit32u get_odd_word () { return (Bit32u)((value>>32) & 0xffffffff); }
+  bx_io_redirect_entry_t(): hi(0), lo(0x10000) {}
+
+  Bit32u get_even_word () const { return lo; }
+  Bit32u get_odd_word () const  { return hi; }
   void set_even_word (Bit32u even) {
     // keep high 32 bits of value, replace low 32
-    value = ((value >> 32) << 32) | (even & 0xffffffff);
+    lo = even; 
     parse_value ();
   }
   void set_odd_word (Bit32u odd) { 
     // keep low 32 bits of value, replace high 32
-    value = (((Bit64u)odd & 0xffffffff) << 32) | (value & 0xffffffff);
+    hi = odd; 
     parse_value ();
   }
   void parse_value ();
+  void sprintf_self (char *buf);
   // parse_value sets the value and all the fields below.  Do not change
   // these fields except by calling parse_value.
   Bit8u dest, masked, trig_mode, remote_irr, polarity, delivery_status, dest_mode, delivery_mode, vector;
-  void sprintf_self (char *buf);
 };
 
 class bx_ioapic_c : public bx_generic_apic_c {
@@ -37,6 +42,7 @@ class bx_ioapic_c : public bx_generic_apic_c {
   // will still be set but delivery will not occur until it is unmasked.
   // It's not clear if this is how the real device works.
   Bit32u irr;
+
 public:
   bx_io_redirect_entry_t ioredtbl[BX_IOAPIC_NUM_PINS];  // table of redirections
   bx_ioapic_c ();
@@ -48,7 +54,5 @@ public:
   void raise_irq (unsigned num, unsigned from);
   void lower_irq (unsigned num, unsigned from);
   void service_ioapic ();
-  virtual bx_bool match_logical_addr (Bit8u address) { return 0; }
-  virtual bx_bool is_local_apic () { return 0; }
   virtual bx_apic_type_t get_type () { return APIC_TYPE_IOAPIC; }
 };
