@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pit.cc,v 1.19 2004/09/05 10:30:19 vruppert Exp $
+// $Id: pit.cc,v 1.20 2004/12/13 19:10:38 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -718,28 +718,6 @@ bx_pit_c::start(unsigned timerid)
 }
 
 
-
-
-  int
-bx_pit_c::SaveState( class state_file *fd )
-{
-  fd->write_check ("8254 start");
-  fd->write (&BX_PIT_THIS s, sizeof (BX_PIT_THIS s));
-  fd->write_check ("8254 end");
-  return(0);
-}
-
-
-  int
-bx_pit_c::LoadState( class state_file *fd )
-{
-  fd->read_check ("8254 start");
-  fd->read (&BX_PIT_THIS s, sizeof (BX_PIT_THIS s));
-  fd->read_check ("8254 end");
-  return(0);
-}
-
-
 #if 0
   void
 bx_kbd_port61h_write(Bit8u   value)
