@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pit82c54.cc,v 1.23 2003/06/29 17:24:52 vruppert Exp $
+// $Id: pit82c54.cc,v 1.25 2004/06/19 15:20:13 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 /*
@@ -26,7 +26,7 @@
  *  write?
  */
 
-#include "bochs.h"
+#include "iodev.h"
 #include "pit82c54.h"
 #define LOG_THIS this->
 
@@ -891,3 +891,7 @@ Bit32u pit_82C54::get_next_event_time(void) {
     out=time2;
   return out;
 }
+
+Bit16u pit_82C54::get_inlatch(int counternum) {
+    return counter[counternum].inlatch;
+}
