@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pit_wrap.h,v 1.19 2004/02/01 23:42:04 cbothamy Exp $
+// $Id: pit_wrap.h,v 1.20 2004/12/13 19:10:38 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -53,8 +53,6 @@ public:
   BX_PIT_SMF void reset( unsigned type);
   BX_PIT_SMF bx_bool periodic( Bit32u   usec_delta );
 
-  BX_PIT_SMF int SaveState( class state_file *fd );
-  BX_PIT_SMF int LoadState( class state_file *fd );
   Bit16u get_timer(int Timer) {
       return s.timer.get_inlatch(Timer);
   }
