@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: slowdown_timer.h,v 1.8 2003/08/19 00:10:38 cbothamy Exp $
+// $Id: slowdown_timer.h,v 1.9 2005/10/22 17:31:02 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 
@@ -14,14 +14,13 @@ private:
     int timer_handle;
 
     float MAXmultiplier;
-    Bit64u Q; // (Q (in seconds))
+    Bit64u Q; // sleep rate in usec
   } s;
 
 public:
   bx_slowdown_timer_c();
 
   void init(void);
-  void reset(unsigned type);
 
   static void timer_handler(void * this_ptr);
 
