@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: vga.h,v 1.46 2005/03/18 14:52:29 vruppert Exp $
+// $Id: vga.h,v 1.49 2005/11/27 17:49:59 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -150,14 +150,13 @@ public:
   virtual void   redraw_area(unsigned x0, unsigned y0,
                              unsigned width, unsigned height);
 
-  virtual void   set_update_interval (unsigned interval);
   virtual void   get_text_snapshot(Bit8u **text_snapshot, unsigned *txHeight,
                                    unsigned *txWidth);
   virtual Bit8u  get_actl_palette_idx(Bit8u index);
 
 protected:
   void init_iohandlers(bx_read_handler_t f_read, bx_write_handler_t f_write);
-  void init_systemtimer(bx_timer_handler_t f_timer);
+  void init_systemtimer(bx_timer_handler_t f_timer, param_event_handler f_param);
 
   static Bit32u read_handler(void *this_ptr, Bit32u address, unsigned io_len);
   static void   write_handler(void *this_ptr, Bit32u address, Bit32u value, unsigned io_len);
@@ -277,9 +276,10 @@ protected:
     Bit16u charmap_address;
     bx_bool x_dotclockdiv2;
     bx_bool y_doublescan;
+    Bit8u last_bpp;
 
 #if BX_SUPPORT_VBE    
-    Bit8u vbe_memory[VBE_DISPI_TOTAL_VIDEO_MEMORY_BYTES];
+    Bit8u   *vbe_memory;
     Bit16u  vbe_cur_dispi;
     Bit16u  vbe_xres;
     Bit16u  vbe_yres;
@@ -328,6 +328,7 @@ protected:
   public:
   static void     timer_handler(void *);
   BX_VGA_SMF void timer(void);
+  static Bit64s   vga_param_handler(bx_param_c *param, int set, Bit64s val);
 
   protected:
   BX_VGA_SMF void update(void);
